import numpy as np 
from scipy.optimize import least_squares 
from scipy.optimize import minimize
from scipy.linalg import lstsq as sclstsq

def harmonic2 ( f1, f2, sN, fs, nK, t ): 
    """
    Performs inverse calculation of two harmonics contaminating a signal. 
    Args:
        f01 = base frequency of the first sinusoidal noise 
        f02 = base frequency of the second sinusoidal noise 
        sN = signal containing noise 
        fs = sampling frequency
        nK = number of harmonics to calculate 
        t = time samples 
    """
    print("building matrix ")
    A = np.zeros( (len(t),  4*nK) )
    #f1 = f1MHz * 1e-3
    #f2 = f2MHz * 1e-3
    for irow, tt in enumerate(t): 
        A[irow, 0:2*nK:2] = np.cos( np.arange(nK)*2*np.pi*(f1/fs)*irow )
        A[irow, 1:2*nK:2] = np.sin( np.arange(nK)*2*np.pi*(f1/fs)*irow )
        A[irow, 2*nK::2] = np.cos( np.arange(nK)*2*np.pi*(f2/fs)*irow )
        A[irow, 2*nK+1::2] = np.sin( np.arange(nK)*2*np.pi*(f2/fs)*irow )

    v = np.linalg.lstsq(A, sN, rcond=1e-8)
    #v = sclstsq(A, sN) #, rcond=1e-6)

    alpha = v[0][0:2*nK:2]
    beta  = v[0][1:2*nK:2]
    amp = np.sqrt( alpha**2 + beta**2 )
    phase = np.arctan(- beta/alpha)
    
    alpha2 = v[0][2*nK::2]
    beta2  = v[0][2*nK+1::2]
    amp2 = np.sqrt( alpha2**2 + beta2**2 )
    phase2 = np.arctan(- beta2/alpha2)

    h = np.zeros(len(t))
    for ik in range(nK):
        h += np.sqrt(alpha[ik]**2 + beta[ik]**2) * np.cos( 2.*np.pi*ik * (f1/fs) * np.arange(0, len(t), 1 )  + phase[ik] ) \
           + np.sqrt(alpha2[ik]**2 + beta2[ik]**2) * np.cos( 2.*np.pi*ik * (f2/fs) * np.arange(0, len(t), 1 )  + phase2[ik] )

    return sN-h

def harmonic ( f0, sN, fs, nK, t ): 
    """
    Performs inverse calculation of harmonics contaminating a signal. 
    Args:
        f0 = base frequency of the sinusoidal noise 
        sN = signal containing noise 
        fs = sampling frequency
        nK = number of harmonics to calculate 
        t = time samples 
    """
    print("building matrix ")
    A = np.zeros( (len(t),  2*nK) )
    for irow, tt in enumerate(t): 
        A[irow, 0::2] = np.cos( np.arange(nK)*2*np.pi*(f0/fs)*irow )
        A[irow, 1::2] = np.sin( np.arange(nK)*2*np.pi*(f0/fs)*irow )

    v = np.linalg.lstsq(A, sN, rcond=None) #, rcond=1e-8)

    alpha = v[0][0::2]
    beta  = v[0][1::2]

    amp = np.sqrt( alpha**2 + beta**2 )
    phase = np.arctan(- beta/alpha)

    #print("amp:", amp, " phase", phase)

    h = np.zeros(len(t))
    for ik in range(nK):
        h += np.sqrt(alpha[ik]**2 + beta[ik]**2) * np.cos( 2.*np.pi*ik * (f0/fs) * np.arange(0, len(t), 1 )  + phase[ik] )

    #plt.matshow(A, aspect='auto')
    #plt.colorbar()

    #plt.figure()
    #plt.plot(alpha)
    #plt.plot(beta)
    #plt.plot(amp)

    #plt.figure()
    #plt.plot(h)
    #plt.title("modelled noise")
    return sN-h

def jacobian( f0, sN, fs, nK, t):
    print("building Jacobian matrix ")
    A = np.zeros( (len(t),  2*nK) )
    for irow, tt in enumerate(t): 
        #A[irow, 0::2] = np.cos( np.arange(nK)*2*np.pi*(f0/fs)*irow )
        #A[irow, 1::2] = np.sin( np.arange(nK)*2*np.pi*(f0/fs)*irow )
        # brutal 
        for k, ik in enumerate( np.arange(0, 2*nK, 2) ):
            #A[irow, ik  ] = np.cos( k*2*np.pi*(f0/fs)*irow )
            #A[irow, ik+1] = np.sin( k*2*np.pi*(f0/fs)*irow )    
            A[irow, ik  ] = - (2.*np.pi*k*irow * sin((2.*np.pi*irow*f0)/fs)) / fs
            A[irow, ik+1] =   (2.*np.pi*k*irow * cos((2.*np.pi*irow*f0)/fs)) / fs


def harmonicNorm ( f0, sN, fs, nK, t ): 
    return np.linalg.norm( harmonic(f0, sN, fs, nK, t))

def harmonic2Norm ( f0, sN, fs, nK, t ): 
    return np.linalg.norm(harmonic2(f0[0], f0[1], sN, fs, nK, t))

def minHarmonic(f0, sN, fs, nK, t):
    f02 = guessf0(sN, fs)
    print("minHarmonic", f0, fs, nK, " guess=", f02)
    res = minimize( harmonicNorm, np.array((f0)), args=(sN, fs, nK, t)) #, method='Nelder-Mead' )# jac=None, hess=None, bounds=None )
    print(res)
    return harmonic(res.x[0], sN, fs, nK, t)

def minHarmonic2(f1, f2, sN, fs, nK, t):
    #f02 = guessf0(sN, fs)
    #print("minHarmonic2", f0, fs, nK, " guess=", f02)
    #methods with bounds, L-BFGS-B, TNC, SLSQP
    res = minimize( harmonic2Norm, np.array((f1,f2)), args=(sN, fs, nK, t)) #, bounds=((f1-1.,f1+1.0),(f2-1.0,f2+1.0)), method='SLSQP' )
    print(res)
    return harmonic2(res.x[0], res.x[1], sN, fs, nK, t) 

def guessf0( sN, fs ):
    S = np.fft.fft(sN)
    w = np.fft.fftfreq( len(sN), 1/fs )
    imax = np.argmax( np.abs(S) )
    #plt.plot( w, np.abs(S) )
    #plt.show()
    #print(w)
    #print ( w[imax], w[imax+1] )
    return abs(w[imax])

if __name__ == "__main__":
    
    import matplotlib.pyplot as plt 

    f0 = 60      # Hz
    f1 = 60      # Hz
    delta  = 0 #np.random.rand() 
    delta2 = 0 #np.random.rand() 
    print("delta", delta)
    fs = 10000   # GMR 
    t = np.arange(0, 1, 1/fs)
    phi = 0 #np.random.rand() 
    phi2 = 0 # np.random.rand() 
    A =  1.0
    A2 = 0.0 
    nK = 10
    T2 = .200
    sN  = A * np.sin( ( 1*(delta  +f0))*2*np.pi*t + phi ) + \
          A2* np.sin( ( 1*(delta2 +f1))*2*np.pi*t + phi2 ) + \
              np.random.normal(0,.1,len(t)) + \
              + np.exp( -t/T2  ) 

    sNc = A * np.sin(  (1*(delta +f0))*2*np.pi*t + phi ) + \
          A2* np.sin(  (1*(delta2+f1))*2*np.pi*t + phi2 ) + \
              + np.exp( -t/T2  ) 


    guessf0(sN, fs)

    #h = harmonic( f0, sN, fs, nK, t) 
    #h = minHarmonic2( f0, f1, sN, fs, nK, t) 
    h = harmonic2( f0, f1, sN, fs, nK, t) 

    plt.figure()
    plt.plot(t, sN, label="sN")
    #plt.plot(t, sN-h, label="sN-h")
    plt.plot(t, h, label='h')
    plt.title("harmonic")
    plt.legend()

    plt.figure()
    plt.plot(t, sN-sNc, label='true noise')
    plt.plot(t, h, label='harmonic removal')
    plt.plot(t, np.exp(-t/T2), label="nmr")
    plt.legend()
    plt.title("true noise")
    
    plt.show()




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main.ui'
#
# Created: Fri Jan 11 23:34:58 2013
#      by: PyQt4 UI code generator 4.9.6
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(1000, 980)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Maximum, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(60, 60))
        MainWindow.setMaximumSize(QtCore.QSize(1000, 16777215))
        MainWindow.setWindowOpacity(1.0)
        MainWindow.setAutoFillBackground(True)
        self.centralwidget = QtGui.QWidget(MainWindow)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setMinimumSize(QtCore.QSize(0, 0))
        self.centralwidget.setMaximumSize(QtCore.QSize(1000, 16777215))
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.scrollArea = QtGui.QScrollArea(self.centralwidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName(_fromUtf8("scrollArea"))
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 986, 927))
        self.scrollAreaWidgetContents.setObjectName(_fromUtf8("scrollAreaWidgetContents"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.tabWidget = QtGui.QTabWidget(self.scrollAreaWidgetContents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setMinimumSize(QtCore.QSize(940, 0))
        self.tabWidget.setObjectName(_fromUtf8("tabWidget"))
        self.tab = QtGui.QWidget()
        self.tab.setObjectName(_fromUtf8("tab"))
        self.gridLayout = QtGui.QGridLayout(self.tab)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.groupBox_4 = QtGui.QGroupBox(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_4.sizePolicy().hasHeightForWidth())
        self.groupBox_4.setSizePolicy(sizePolicy)
        self.groupBox_4.setMinimumSize(QtCore.QSize(480, 150))
        self.groupBox_4.setFlat(False)
        self.groupBox_4.setCheckable(True)
        self.groupBox_4.setObjectName(_fromUtf8("groupBox_4"))
        self.gridLayout.addWidget(self.groupBox_4, 4, 1, 1, 1)
        self.groupBox = QtGui.QGroupBox(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMinimumSize(QtCore.QSize(460, 230))
        self.groupBox.setAutoFillBackground(False)
        self.groupBox.setFlat(False)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.headerFileTextBrowser = QtGui.QTextBrowser(self.groupBox)
        self.headerFileTextBrowser.setGeometry(QtCore.QRect(10, 30, 441, 31))
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.headerFileTextBrowser.sizePolicy().hasHeightForWidth())
        self.headerFileTextBrowser.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setItalic(True)
        self.headerFileTextBrowser.setFont(font)
        self.headerFileTextBrowser.setObjectName(_fromUtf8("headerFileTextBrowser"))
        self.label_4 = QtGui.QLabel(self.groupBox)
        self.label_4.setGeometry(QtCore.QRect(10, 70, 81, 21))
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.pulseTypeTextBrowser = QtGui.QTextBrowser(self.groupBox)
        self.pulseTypeTextBrowser.setGeometry(QtCore.QRect(90, 70, 361, 21))
        font = QtGui.QFont()
        font.setItalic(True)
        self.pulseTypeTextBrowser.setFont(font)
        self.pulseTypeTextBrowser.setAcceptDrops(True)
        self.pulseTypeTextBrowser.setObjectName(_fromUtf8("pulseTypeTextBrowser"))
        self.lcdNumberNuTx = QtGui.QLCDNumber(self.groupBox)
        self.lcdNumberNuTx.setGeometry(QtCore.QRect(160, 100, 64, 23))
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lcdNumberNuTx.sizePolicy().hasHeightForWidth())
        self.lcdNumberNuTx.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.lcdNumberNuTx.setFont(font)
        self.lcdNumberNuTx.setWhatsThis(_fromUtf8(""))
        self.lcdNumberNuTx.setAutoFillBackground(False)
        self.lcdNumberNuTx.setStyleSheet(_fromUtf8("#lcdNumberNuTx {\n"
"color: green;\n"
"background: black;\n"
"}\n"
"\n"
"#lcdNumberNuTx:disabled {\n"
"color: grey;\n"
"background: black;\n"
"}"))
        self.lcdNumberNuTx.setFrameShadow(QtGui.QFrame.Raised)
        self.lcdNumberNuTx.setLineWidth(1)
        self.lcdNumberNuTx.setMidLineWidth(0)
        self.lcdNumberNuTx.setSegmentStyle(QtGui.QLCDNumber.Flat)
        self.lcdNumberNuTx.setProperty("value", 0.0)
        self.lcdNumberNuTx.setObjectName(_fromUtf8("lcdNumberNuTx"))
        self.lcdNumberTuneuF = QtGui.QLCDNumber(self.groupBox)
        self.lcdNumberTuneuF.setGeometry(QtCore.QRect(370, 100, 64, 23))
        self.lcdNumberTuneuF.setStyleSheet(_fromUtf8("#lcdNumberTuneuF {\n"
"color: green;\n"
"background: black;\n"
"}\n"
"\n"
"#lcdNumberTuneuF:disabled {\n"
"color: grey;\n"
"background: black;\n"
"}"))
        self.lcdNumberTuneuF.setLineWidth(1)
        self.lcdNumberTuneuF.setMidLineWidth(0)
        self.lcdNumberTuneuF.setSegmentStyle(QtGui.QLCDNumber.Flat)
        self.lcdNumberTuneuF.setObjectName(_fromUtf8("lcdNumberTuneuF"))
        self.lcdNumberTauPulse1 = QtGui.QLCDNumber(self.groupBox)
        self.lcdNumberTauPulse1.setGeometry(QtCore.QRect(160, 130, 64, 23))
        self.lcdNumberTauPulse1.setStyleSheet(_fromUtf8("#lcdNumberTauPulse1 {\n"
"color: green;\n"
"background: black;\n"
"}\n"
"\n"
"#lcdNumberTauPulse1:disabled {\n"
"color: grey;\n"
"background: black;\n"
"}"))
        self.lcdNumberTauPulse1.setFrameShadow(QtGui.QFrame.Raised)
        self.lcdNumberTauPulse1.setLineWidth(1)
        self.lcdNumberTauPulse1.setMidLineWidth(0)
        self.lcdNumberTauPulse1.setSegmentStyle(QtGui.QLCDNumber.Flat)
        self.lcdNumberTauPulse1.setObjectName(_fromUtf8("lcdNumberTauPulse1"))
        self.label_6 = QtGui.QLabel(self.groupBox)
        self.label_6.setGeometry(QtCore.QRect(10, 100, 91, 21))
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.label_7 = QtGui.QLabel(self.groupBox)
        self.label_7.setGeometry(QtCore.QRect(10, 130, 141, 21))
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self.label_13 = QtGui.QLabel(self.groupBox)
        self.label_13.setGeometry(QtCore.QRect(260, 160, 91, 21))
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.lcdNumberTauPulse2 = QtGui.QLCDNumber(self.groupBox)
        self.lcdNumberTauPulse2.setGeometry(QtCore.QRect(160, 160, 64, 23))
        self.lcdNumberTauPulse2.setStyleSheet(_fromUtf8("#lcdNumberTauPulse2 {\n"
"color: green;\n"
"background: black;\n"
"}\n"
"\n"
"#lcdNumberTauPulse2:disabled{\n"
"color: grey;\n"
"background: black;\n"
"}"))
        self.lcdNumberTauPulse2.setLineWidth(1)
        self.lcdNumberTauPulse2.setMidLineWidth(0)
        self.lcdNumberTauPulse2.setSegmentStyle(QtGui.QLCDNumber.Flat)
        self.lcdNumberTauPulse2.setObjectName(_fromUtf8("lcdNumberTauPulse2"))
        self.label_14 = QtGui.QLabel(self.groupBox)
        self.label_14.setGeometry(QtCore.QRect(260, 100, 111, 21))
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.label_15 = QtGui.QLabel(self.groupBox)
        self.label_15.setGeometry(QtCore.QRect(260, 130, 111, 21))
        self.label_15.setObjectName(_fromUtf8("label_15"))
        self.lcdNumberSampFreq = QtGui.QLCDNumber(self.groupBox)
        self.lcdNumberSampFreq.setEnabled(True)
        self.lcdNumberSampFreq.setGeometry(QtCore.QRect(370, 130, 64, 23))
        self.lcdNumberSampFreq.setStyleSheet(_fromUtf8("#lcdNumberSampFreq {\n"
"color: green;\n"
"background: black;\n"
"}\n"
"\n"
"#lcdNumberSampFreq:disabled{\n"
"color: grey;\n"
"background: black;\n"
"}"))
        self.lcdNumberSampFreq.setLineWidth(1)
        self.lcdNumberSampFreq.setMidLineWidth(0)
        self.lcdNumberSampFreq.setDigitCount(5)
        self.lcdNumberSampFreq.setSegmentStyle(QtGui.QLCDNumber.Flat)
        self.lcdNumberSampFreq.setObjectName(_fromUtf8("lcdNumberSampFreq"))
        self.lcdNumberTauDelay = QtGui.QLCDNumber(self.groupBox)
        self.lcdNumberTauDelay.setEnabled(True)
        self.lcdNumberTauDelay.setGeometry(QtCore.QRect(370, 160, 64, 23))
        self.lcdNumberTauDelay.setStyleSheet(_fromUtf8("#lcdNumberTauDelay {\n"
"color: green;\n"
"background: black;\n"
"}\n"
"\n"
"#lcdNumberTauDelay:disabled {\n"
"color: grey;\n"
"background: black;\n"
"}"))
        self.lcdNumberTauDelay.setLineWidth(1)
        self.lcdNumberTauDelay.setMidLineWidth(0)
        self.lcdNumberTauDelay.setSegmentStyle(QtGui.QLCDNumber.Flat)
        self.lcdNumberTauDelay.setObjectName(_fromUtf8("lcdNumberTauDelay"))
        self.label_16 = QtGui.QLabel(self.groupBox)
        self.label_16.setGeometry(QtCore.QRect(10, 160, 131, 21))
        self.label_16.setObjectName(_fromUtf8("label_16"))
        self.lcdNumberNQ = QtGui.QLCDNumber(self.groupBox)
        self.lcdNumberNQ.setGeometry(QtCore.QRect(160, 190, 64, 23))
        self.lcdNumberNQ.setStyleSheet(_fromUtf8("#lcdNumberNQ {\n"
"color: green;\n"
"background: black;\n"
"}\n"
"\n"
"#lcdNumberNQ:disabled{\n"
"color: grey;\n"
"background: black;\n"
"}"))
        self.lcdNumberNQ.setSegmentStyle(QtGui.QLCDNumber.Flat)
        self.lcdNumberNQ.setObjectName(_fromUtf8("lcdNumberNQ"))
        self.label_9 = QtGui.QLabel(self.groupBox)
        self.label_9.setGeometry(QtCore.QRect(10, 190, 141, 21))
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.gridLayout.addWidget(self.groupBox, 4, 0, 1, 1)
        self.groupBox_10 = QtGui.QGroupBox(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_10.sizePolicy().hasHeightForWidth())
        self.groupBox_10.setSizePolicy(sizePolicy)
        self.groupBox_10.setMinimumSize(QtCore.QSize(460, 160))
        self.groupBox_10.setObjectName(_fromUtf8("groupBox_10"))
        self.stacksLineEdit = QtGui.QLineEdit(self.groupBox_10)
        self.stacksLineEdit.setGeometry(QtCore.QRect(110, 30, 91, 21))
        self.stacksLineEdit.setToolTip(_fromUtf8("<html><head/><body><p>Set the stacks that you would like processed.</p><p>This must be a valid set of numpy array indices. Remember that Python uses non end-inclusive indexing. </p><p>So things like [1:24] will include stacks 1-23</p><p>Furthermore [1:8,12:24] will include stacks 1-7 and 12:23. Any list of valid indices will be accepted, but they must be comma seperated. </p></body></html>"))
        self.stacksLineEdit.setObjectName(_fromUtf8("stacksLineEdit"))
        self.label = QtGui.QLabel(self.groupBox_10)
        self.label.setGeometry(QtCore.QRect(10, 30, 51, 20))
        self.label.setToolTip(_fromUtf8(""))
        self.label.setStatusTip(_fromUtf8(""))
        self.label.setObjectName(_fromUtf8("label"))
        self.label_23 = QtGui.QLabel(self.groupBox_10)
        self.label_23.setGeometry(QtCore.QRect(10, 65, 101, 16))
        self.label_23.setObjectName(_fromUtf8("label_23"))
        self.dataChanLineEdit = QtGui.QLineEdit(self.groupBox_10)
        self.dataChanLineEdit.setGeometry(QtCore.QRect(110, 60, 91, 21))
        self.dataChanLineEdit.setObjectName(_fromUtf8("dataChanLineEdit"))
        self.label_24 = QtGui.QLabel(self.groupBox_10)
        self.label_24.setGeometry(QtCore.QRect(220, 36, 101, 16))
        self.label_24.setObjectName(_fromUtf8("label_24"))
        self.DeadTimeSpinBox = QtGui.QDoubleSpinBox(self.groupBox_10)
        self.DeadTimeSpinBox.setGeometry(QtCore.QRect(360, 30, 91, 25))
        self.DeadTimeSpinBox.setMinimum(5.0)
        self.DeadTimeSpinBox.setSingleStep(0.5)
        self.DeadTimeSpinBox.setProperty("value", 5.0)
        self.DeadTimeSpinBox.setObjectName(_fromUtf8("DeadTimeSpinBox"))
        self.label_28 = QtGui.QLabel(self.groupBox_10)
        self.label_28.setGeometry(QtCore.QRect(220, 64, 131, 16))
        self.label_28.setObjectName(_fromUtf8("label_28"))
        self.refChanLineEdit = QtGui.QLineEdit(self.groupBox_10)
        self.refChanLineEdit.setGeometry(QtCore.QRect(360, 60, 91, 21))
        self.refChanLineEdit.setObjectName(_fromUtf8("refChanLineEdit"))
        self.label_30 = QtGui.QLabel(self.groupBox_10)
        self.label_30.setGeometry(QtCore.QRect(12, 96, 91, 16))
        self.label_30.setObjectName(_fromUtf8("label_30"))
        self.CentralVSpinBox = QtGui.QDoubleSpinBox(self.groupBox_10)
        self.CentralVSpinBox.setGeometry(QtCore.QRect(110, 90, 91, 25))
        self.CentralVSpinBox.setMinimum(100.0)
        self.CentralVSpinBox.setMaximum(5000.99)
        self.CentralVSpinBox.setSingleStep(1.0)
        self.CentralVSpinBox.setObjectName(_fromUtf8("CentralVSpinBox"))
        self.label_29 = QtGui.QLabel(self.groupBox_10)
        self.label_29.setGeometry(QtCore.QRect(220, 95, 111, 16))
        self.label_29.setObjectName(_fromUtf8("label_29"))
        self.FIDProcComboBox = QtGui.QComboBox(self.groupBox_10)
        self.FIDProcComboBox.setGeometry(QtCore.QRect(360, 90, 91, 25))
        self.FIDProcComboBox.setObjectName(_fromUtf8("FIDProcComboBox"))
        self.fullWorkflowPushButton = QtGui.QPushButton(self.groupBox_10)
        self.fullWorkflowPushButton.setGeometry(QtCore.QRect(280, 130, 131, 31))
        self.fullWorkflowPushButton.setAutoFillBackground(False)
        self.fullWorkflowPushButton.setStyleSheet(_fromUtf8("#fullWorkflowPushButton {\n"
"background: green;\n"
"}"))
        self.fullWorkflowPushButton.setAutoDefault(False)
        self.fullWorkflowPushButton.setDefault(False)
        self.fullWorkflowPushButton.setFlat(False)
        self.fullWorkflowPushButton.setObjectName(_fromUtf8("fullWorkflowPushButton"))
        self.loadDataPushButton = QtGui.QPushButton(self.groupBox_10)
        self.loadDataPushButton.setGeometry(QtCore.QRect(90, 130, 91, 31))
        self.loadDataPushButton.setObjectName(_fromUtf8("loadDataPushButton"))
        self.gridLayout.addWidget(self.groupBox_10, 5, 0, 1, 1)
        self.groupBox_9 = QtGui.QGroupBox(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_9.sizePolicy().hasHeightForWidth())
        self.groupBox_9.setSizePolicy(sizePolicy)
        self.groupBox_9.setMinimumSize(QtCore.QSize(480, 100))
        self.groupBox_9.setCheckable(True)
        self.groupBox_9.setObjectName(_fromUtf8("groupBox_9"))
        self.checkBox = QtGui.QCheckBox(self.groupBox_9)
        self.checkBox.setGeometry(QtCore.QRect(30, 30, 171, 20))
        self.checkBox.setObjectName(_fromUtf8("checkBox"))
        self.comboBox_2 = QtGui.QComboBox(self.groupBox_9)
        self.comboBox_2.setGeometry(QtCore.QRect(130, 50, 78, 25))
        self.comboBox_2.setObjectName(_fromUtf8("comboBox_2"))
        self.label_5 = QtGui.QLabel(self.groupBox_9)
        self.label_5.setGeometry(QtCore.QRect(40, 60, 71, 16))
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.doubleSpinBox = QtGui.QDoubleSpinBox(self.groupBox_9)
        self.doubleSpinBox.setGeometry(QtCore.QRect(390, 20, 62, 25))
        self.doubleSpinBox.setObjectName(_fromUtf8("doubleSpinBox"))
        self.label_8 = QtGui.QLabel(self.groupBox_9)
        self.label_8.setGeometry(QtCore.QRect(220, 20, 161, 20))
        self.label_8.setObjectName(_fromUtf8("label_8"))
        self.gridLayout.addWidget(self.groupBox_9, 5, 1, 1, 1)
        self.groupBox_7 = QtGui.QGroupBox(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_7.sizePolicy().hasHeightForWidth())
        self.groupBox_7.setSizePolicy(sizePolicy)
        self.groupBox_7.setMinimumSize(QtCore.QSize(480, 100))
        self.groupBox_7.setCheckable(True)
        self.groupBox_7.setObjectName(_fromUtf8("groupBox_7"))
        self.comboBox = QtGui.QComboBox(self.groupBox_7)
        self.comboBox.setGeometry(QtCore.QRect(40, 30, 141, 22))
        self.comboBox.setMouseTracking(True)
        self.comboBox.setToolTip(_fromUtf8(""))
        self.comboBox.setStatusTip(_fromUtf8(""))
        self.comboBox.setWhatsThis(_fromUtf8(""))
        self.comboBox.setObjectName(_fromUtf8("comboBox"))
        self.comboBox.addItem(_fromUtf8(""))
        self.comboBox.addItem(_fromUtf8(""))
        self.gridLayout.addWidget(self.groupBox_7, 2, 1, 1, 1)
        self.label_2 = QtGui.QLabel(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 0, 1, 1, 1)
        self.groupBox_8 = QtGui.QGroupBox(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_8.sizePolicy().hasHeightForWidth())
        self.groupBox_8.setSizePolicy(sizePolicy)
        self.groupBox_8.setMinimumSize(QtCore.QSize(480, 150))
        self.groupBox_8.setCheckable(True)
        self.groupBox_8.setObjectName(_fromUtf8("groupBox_8"))
        self.label_3 = QtGui.QLabel(self.groupBox_8)
        self.label_3.setGeometry(QtCore.QRect(30, 60, 111, 16))
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.spinBox = QtGui.QSpinBox(self.groupBox_8)
        self.spinBox.setGeometry(QtCore.QRect(140, 60, 101, 25))
        self.spinBox.setMaximum(1000)
        self.spinBox.setProperty("value", 300)
        self.spinBox.setObjectName(_fromUtf8("spinBox"))
        self.gridLayout.addWidget(self.groupBox_8, 3, 1, 1, 1)
        self.groupBox_5 = QtGui.QGroupBox(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
        self.groupBox_5.setSizePolicy(sizePolicy)
        self.groupBox_5.setMinimumSize(QtCore.QSize(480, 150))
        self.groupBox_5.setCheckable(True)
        self.groupBox_5.setObjectName(_fromUtf8("groupBox_5"))
        self.gridLayout.addWidget(self.groupBox_5, 1, 1, 1, 1)
        self.mplwidget = MyDynamicMplCanvas(self.tab)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mplwidget.sizePolicy().hasHeightForWidth())
        self.mplwidget.setSizePolicy(sizePolicy)
        self.mplwidget.setMinimumSize(QtCore.QSize(460, 460))
        self.mplwidget.setObjectName(_fromUtf8("mplwidget"))
        self.gridLayout.addWidget(self.mplwidget, 0, 0, 4, 1)
        self.tabWidget.addTab(self.tab, _fromUtf8(""))
        self.tab_3 = QtGui.QWidget()
        self.tab_3.setObjectName(_fromUtf8("tab_3"))
        self.treeWidget = QtGui.QTreeWidget(self.tab_3)
        self.treeWidget.setGeometry(QtCore.QRect(70, 50, 256, 192))
        self.treeWidget.setObjectName(_fromUtf8("treeWidget"))
        self.treeWidget.headerItem().setText(0, _fromUtf8("1"))
        self.toolBox = QtGui.QToolBox(self.tab_3)
        self.toolBox.setGeometry(QtCore.QRect(320, 460, 79, 137))
        self.toolBox.setObjectName(_fromUtf8("toolBox"))
        self.page = QtGui.QWidget()
        self.page.setGeometry(QtCore.QRect(0, 0, 94, 24))
        self.page.setObjectName(_fromUtf8("page"))
        self.toolBox.addItem(self.page, _fromUtf8(""))
        self.page_2 = QtGui.QWidget()
        self.page_2.setGeometry(QtCore.QRect(0, 0, 94, 24))
        self.page_2.setObjectName(_fromUtf8("page_2"))
        self.toolBox.addItem(self.page_2, _fromUtf8(""))
        self.dateEdit = QtGui.QDateEdit(self.tab_3)
        self.dateEdit.setGeometry(QtCore.QRect(400, 280, 110, 25))
        self.dateEdit.setObjectName(_fromUtf8("dateEdit"))
        self.tabWidget.addTab(self.tab_3, _fromUtf8(""))
        self.tab_2 = QtGui.QWidget()
        self.tab_2.setObjectName(_fromUtf8("tab_2"))
        self.dateEdit_2 = QtGui.QDateEdit(self.tab_2)
        self.dateEdit_2.setGeometry(QtCore.QRect(50, 100, 110, 25))
        self.dateEdit_2.setObjectName(_fromUtf8("dateEdit_2"))
        self.tabWidget.addTab(self.tab_2, _fromUtf8(""))
        self.tab_6 = QtGui.QWidget()
        self.tab_6.setObjectName(_fromUtf8("tab_6"))
        self.tabWidget.addTab(self.tab_6, _fromUtf8(""))
        self.tab_4 = QtGui.QWidget()
        self.tab_4.setObjectName(_fromUtf8("tab_4"))
        self.invertButton = QtGui.QPushButton(self.tab_4)
        self.invertButton.setGeometry(QtCore.QRect(290, 140, 311, 141))
        self.invertButton.setStyleSheet(_fromUtf8("#invertButton {\n"
"font-size:29pt;\n"
"font-weight: bold;\n"
"color: white;\n"
"background: red;\n"
"}"))
        self.invertButton.setObjectName(_fromUtf8("invertButton"))
        self.tabWidget.addTab(self.tab_4, _fromUtf8(""))
        self.tab_5 = QtGui.QWidget()
        self.tab_5.setObjectName(_fromUtf8("tab_5"))
        self.tabWidget.addTab(self.tab_5, _fromUtf8(""))
        self.horizontalLayout_2.addWidget(self.tabWidget)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.horizontalLayout.addWidget(self.scrollArea)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1000, 20))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName(_fromUtf8("menuFile"))
        self.menuAbout = QtGui.QMenu(self.menubar)
        self.menuAbout.setObjectName(_fromUtf8("menuAbout"))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)
        self.actionClose = QtGui.QAction(MainWindow)
        self.actionClose.setObjectName(_fromUtf8("actionClose"))
        self.actionAboutBrewCentral = QtGui.QAction(MainWindow)
        self.actionAboutBrewCentral.setObjectName(_fromUtf8("actionAboutBrewCentral"))
        self.actionNothing = QtGui.QAction(MainWindow)
        self.actionNothing.setObjectName(_fromUtf8("actionNothing"))
        self.actionTemperature = QtGui.QAction(MainWindow)
        self.actionTemperature.setObjectName(_fromUtf8("actionTemperature"))
        self.actionOpen_GMR = QtGui.QAction(MainWindow)
        self.actionOpen_GMR.setCheckable(False)
        self.actionOpen_GMR.setObjectName(_fromUtf8("actionOpen_GMR"))
        self.actionProcess = QtGui.QAction(MainWindow)
        self.actionProcess.setCheckable(True)
        self.actionProcess.setObjectName(_fromUtf8("actionProcess"))
        self.actionOpen_Preprocessed_dataset = QtGui.QAction(MainWindow)
        self.actionOpen_Preprocessed_dataset.setEnabled(False)
        self.actionOpen_Preprocessed_dataset.setObjectName(_fromUtf8("actionOpen_Preprocessed_dataset"))
        self.actionOpen_VC_Preprocessed_dataset = QtGui.QAction(MainWindow)
        self.actionOpen_VC_Preprocessed_dataset.setEnabled(False)
        self.actionOpen_VC_Preprocessed_dataset.setObjectName(_fromUtf8("actionOpen_VC_Preprocessed_dataset"))
        self.actionSave_Preprocesssed_Dataset = QtGui.QAction(MainWindow)
        self.actionSave_Preprocesssed_Dataset.setEnabled(False)
        self.actionSave_Preprocesssed_Dataset.setObjectName(_fromUtf8("actionSave_Preprocesssed_Dataset"))
        self.menuFile.addAction(self.actionOpen_GMR)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionOpen_Preprocessed_dataset)
        self.menuFile.addAction(self.actionOpen_VC_Preprocessed_dataset)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSave_Preprocesssed_Dataset)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionClose)
        self.menuAbout.addAction(self.actionAboutBrewCentral)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuAbout.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        self.toolBox.setCurrentIndex(0)
        QtCore.QObject.connect(self.actionClose, QtCore.SIGNAL(_fromUtf8("activated()")), MainWindow.close)
        QtCore.QObject.connect(self.actionAboutBrewCentral, QtCore.SIGNAL(_fromUtf8("activated()")), MainWindow.show)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "Avko - sNMR Workbench", None))
        self.groupBox_4.setTitle(_translate("MainWindow", "Adaptive Noise Suppresion", None))
        self.groupBox.setTitle(_translate("MainWindow", "Header file info", None))
        self.headerFileTextBrowser.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:8pt; font-weight:400; font-style:italic;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'DejaVu Serif\'; font-size:9pt;\">Load supported RAW Dataset header from file menu</span></p></body></html>", None))
        self.label_4.setText(_translate("MainWindow", "Pulse Type", None))
        self.pulseTypeTextBrowser.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:italic;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'DejaVu Serif\'; font-size:10pt;\"><br /></p></body></html>", None))
        self.label_6.setText(_translate("MainWindow", "ν Tx  [Hz]", None))
        self.label_7.setText(_translate("MainWindow", "τ Pulse 1 [ms]", None))
        self.label_13.setText(_translate("MainWindow", "τ Delay [ms]", None))
        self.label_14.setText(_translate("MainWindow", "Tx tuning [μF]", None))
        self.label_15.setText(_translate("MainWindow", "ν Sampling [Hz]", None))
        self.label_16.setText(_translate("MainWindow", "τ Pulse 2 [ms]", None))
        self.label_9.setText(_translate("MainWindow", "Num. pulse moments", None))
        self.groupBox_10.setTitle(_translate("MainWindow", "Input parameters", None))
        self.label.setText(_translate("MainWindow", "Stacks", None))
        self.label_23.setText(_translate("MainWindow", "Data Channels", None))
        self.label_24.setText(_translate("MainWindow", "Dead time [ms]", None))
        self.label_28.setText(_translate("MainWindow", "Reference Channels", None))
        self.label_30.setText(_translate("MainWindow", "Central ν Hz", None))
        self.CentralVSpinBox.setToolTip(_translate("MainWindow", "<html><head/><body><p>In case of off-resonant transmitter pulse, you can set the central frequency that will be used for all processing. This has the biggest impact on the band-pass filter, and the frequencies used in inversion. </p></body></html>", None))
        self.label_29.setText(_translate("MainWindow", "Process FID #", None))
        self.FIDProcComboBox.setToolTip(_translate("MainWindow", "<html><head/><body><p>For T1 or CPMG pulses, which pulse(s) would you like to process. Note that for very short delay T1 pulses, the first pulse may be disabled. </p></body></html>", None))
        self.fullWorkflowPushButton.setText(_translate("MainWindow", "Full Workflow", None))
        self.loadDataPushButton.setText(_translate("MainWindow", "Load Data", None))
        self.groupBox_9.setTitle(_translate("MainWindow", "SmartStack^TM", None))
        self.checkBox.setText(_translate("MainWindow", "Correct phase jitter", None))
        self.label_5.setText(_translate("MainWindow", "Outlier test", None))
        self.label_8.setText(_translate("MainWindow", "Instrument phase delay", None))
        self.groupBox_7.setTitle(_translate("MainWindow", "Band-Pass Filter", None))
        self.comboBox.setAccessibleDescription(_translate("MainWindow", "Hello", None))
        self.comboBox.setItemText(0, _translate("MainWindow", "Butterworth", None))
        self.comboBox.setItemText(1, _translate("MainWindow", "Chebychev Type II", None))
        self.label_2.setText(_translate("MainWindow", "         Preprocessing Workflow", None))
        self.groupBox_8.setTitle(_translate("MainWindow", "Downsample and truncate", None))
        self.label_3.setText(_translate("MainWindow", "Truncate [ms]", None))
        self.groupBox_5.setTitle(_translate("MainWindow", "Despike Filter", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "Preprocess RAW", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page), _translate("MainWindow", "Page 1", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page_2), _translate("MainWindow", "Page 2", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), _translate("MainWindow", "Data QC", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("MainWindow", "Model Parameters", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_6), _translate("MainWindow", "Forward modelling", None))
        self.invertButton.setText(_translate("MainWindow", "Invert", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_4), _translate("MainWindow", "Inversion", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_5), _translate("MainWindow", "Analysis", None))
        self.menuFile.setTitle(_translate("MainWindow", "&File", None))
        self.menuAbout.setTitle(_translate("MainWindow", "About", None))
        self.actionClose.setText(_translate("MainWindow", "&Close", None))
        self.actionAboutBrewCentral.setText(_translate("MainWindow", "Brew Central", None))
        self.actionNothing.setText(_translate("MainWindow", "Nothing", None))
        self.actionTemperature.setText(_translate("MainWindow", "Temperature", None))
        self.actionOpen_GMR.setText(_translate("MainWindow", "Open GMR RAW header", None))
        self.actionOpen_GMR.setIconText(_translate("MainWindow", "Open GMR RAW dataset(s)", None))
        self.actionProcess.setText(_translate("MainWindow", "Process", None))
        self.actionOpen_Preprocessed_dataset.setText(_translate("MainWindow", "Open Avko Preprocessed dataset", None))
        self.actionOpen_VC_Preprocessed_dataset.setText(_translate("MainWindow", "Open VC Preprocessed dataset", None))
        self.actionSave_Preprocesssed_Dataset.setText(_translate("MainWindow", "Save Preprocesssed Dataset", None))

from mydynamicmplcanvas import MyDynamicMplCanvas



from __future__ import unicode_literals
import sys
import os
import random
import matplotlib
# Make sure that we are using QT5
matplotlib.use('Qt5Agg')
from PyQt5 import QtCore, QtWidgets

from numpy import arange, sin, pi
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import numpy as np

class MyMplCanvas(FigureCanvas):
    
    """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""
    def __init__(self, parent=None, width=3, height=4, dpi=100):
        
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        FigureCanvas.__init__(self, self.fig)

        self.setParent(parent)
        FigureCanvas.updateGeometry(self)

    def compute_initial_figure(self):
        pass

    def clicked(self):
        print ("Clicked")

class MyDynamicMplCanvas(MyMplCanvas):

    """A canvas that updates itself every second with a new plot."""
    def __init__(self, *args, **kwargs):
        MyMplCanvas.__init__(self, *args, **kwargs)
        self.ax1 = self.fig.add_axes([.125,.1,.725,.8])
        self.ax2 = self.ax1.twinx() # fig.add_axes([.125,.1,.725,.8])
        self.compute_initial_figure()

    def reAxH(self, num, shx=True, shy=True):
        try:
            self.fig.clear()
        except:
            pass
        for n in range(num):
            if n == 0:
                self.ax1 = self.fig.add_subplot( 1, num, 1 )
                self.ax1.tick_params(axis='both', which='major', labelsize=8)
                self.ax1.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax1.yaxis.get_offset_text().set_size(8) 
            if n == 1:
                self.ax2 = self.fig.add_subplot( 1, num, 2 )
                self.ax2.tick_params(axis='both', which='major', labelsize=8)
                self.ax2.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax2.yaxis.get_offset_text().set_size(8) 
            if n == 2:
                self.ax3 = self.fig.add_subplot( 1, num, 3 )
                self.ax3.tick_params(axis='both', which='major', labelsize=8)
                self.ax3.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax3.yaxis.get_offset_text().set_size(8) 
            if n == 3:
                self.ax4 = self.fig.add_subplot( 1, num, 4 )
                self.ax4.tick_params(axis='both', which='major', labelsize=8)
                self.ax4.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax4.yaxis.get_offset_text().set_size(8) 
    
    def reAxH2(self, num, shx=True, shy=True):
        try:
            for ax in fig.axes:
                self.fig.delaxes(ax)
        except:
            pass

        try:
            self.fig.clear()
        except:
            pass
        
        for n in range(num):
            if n == 0:
                self.ax1 = self.fig.add_subplot( 2, num, 1 )
                self.ax1.tick_params(axis='both', which='major', labelsize=8)
                self.ax1.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax1.yaxis.get_offset_text().set_size(8) 
                self.ax21 = self.fig.add_subplot( 2, num, num+1 )
                self.ax21.tick_params(axis='both', which='major', labelsize=8)
                self.ax21.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax21.yaxis.get_offset_text().set_size(8) 
            if n == 1:
                self.ax2 = self.fig.add_subplot( 2, num, 2, sharex=self.ax1, sharey=self.ax1 )
                self.ax2.tick_params(axis='both', which='major', labelsize=8)
                self.ax2.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax2.yaxis.get_offset_text().set_size(8) 
                self.ax22 = self.fig.add_subplot( 2, num, num+2, sharex=self.ax21, sharey=self.ax21 )
                self.ax22.tick_params(axis='both', which='major', labelsize=8)
                self.ax22.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax22.yaxis.get_offset_text().set_size(8) 
            if n == 2:
                self.ax3 = self.fig.add_subplot( 2, num, 3, sharex=self.ax1, sharey=self.ax1 )
                self.ax3.tick_params(axis='both', which='major', labelsize=8)
                self.ax3.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax3.yaxis.get_offset_text().set_size(8) 
                self.ax23 = self.fig.add_subplot( 2, num, num+3, sharex=self.ax21, sharey=self.ax21 )
                self.ax23.tick_params(axis='both', which='major', labelsize=8)
                self.ax23.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax23.yaxis.get_offset_text().set_size(8) 
            if n == 3:
                self.ax4 = self.fig.add_subplot( 2, num, 4, sharex=self.ax1, sharey=self.ax1 )
                self.ax4.tick_params(axis='both', which='major', labelsize=8)
                self.ax4.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax4.yaxis.get_offset_text().set_size(8) 
                self.ax24 = self.fig.add_subplot( 2, num, num+4, sharex=self.ax21, sharey=self.ax21 )
                self.ax24.tick_params(axis='both', which='major', labelsize=8)
                self.ax24.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax24.yaxis.get_offset_text().set_size(8) 
            if n == 4:
                self.ax5 = self.fig.add_subplot( 2, num, 5, sharex=self.ax1, sharey=self.ax1 )
                self.ax5.tick_params(axis='both', which='major', labelsize=8)
                self.ax5.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax5.yaxis.get_offset_text().set_size(8) 
                self.ax25 = self.fig.add_subplot( 2, num, num+5, sharex=self.ax21, sharey=self.ax21 )
                self.ax25.tick_params(axis='both', which='major', labelsize=8)
                self.ax25.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax25.yaxis.get_offset_text().set_size(8) 
            if n == 5:
                self.ax6 = self.fig.add_subplot( 2, num, 6, sharex=self.ax1, sharey=self.ax1 )
                self.ax6.tick_params(axis='both', which='major', labelsize=8)
                self.ax6.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax6.yaxis.get_offset_text().set_size(8) 
                self.ax26 = self.fig.add_subplot( 2, num, num+6, sharex=self.ax21, sharey=self.ax21 )
                self.ax26.tick_params(axis='both', which='major', labelsize=8)
                self.ax26.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax26.yaxis.get_offset_text().set_size(8) 
            if n == 6:
                self.ax7 = self.fig.add_subplot( 2, num, 7, sharex=self.ax1, sharey=self.ax1 )
                self.ax7.tick_params(axis='both', which='major', labelsize=8)
                self.ax7.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax7.yaxis.get_offset_text().set_size(8) 
                self.ax27 = self.fig.add_subplot( 2, num, num+7, sharex=self.ax21, sharey=self.ax21 )
                self.ax27.tick_params(axis='both', which='major', labelsize=8)
                self.ax27.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax27.yaxis.get_offset_text().set_size(8) 
            if n == 7:
                self.ax8 = self.fig.add_subplot( 2, num, 8, sharex=self.ax1, sharey=self.ax1 )
                self.ax8.tick_params(axis='both', which='major', labelsize=8)
                self.ax8.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax8.yaxis.get_offset_text().set_size(8) 
                self.ax28 = self.fig.add_subplot( 2, num, num+8, sharex=self.ax21, sharey=self.ax21 )
                self.ax28.tick_params(axis='both', which='major', labelsize=8)
                self.ax28.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax28.yaxis.get_offset_text().set_size(8) 

    def reAx2(self, shx=True, shy=True):

        try:
            self.fig.clear()
        except:
            pass

        try:
            self.ax1.clear() 
            self.delaxes(self.ax1) #.clear()
        except:
            pass
        
        try:    
            self.delaxes(self.ax3) #.clear()
        except:
            pass
        
        try:
            self.ax2.clear() 
            self.delaxes(self.ax2) #.clear()
        except:
            pass

        self.ax1 = self.fig.add_subplot(211)
        if shx and shy:
            self.ax2 = self.fig.add_subplot(212, sharex=self.ax1, sharey=self.ax1)
        elif shx == True:
            self.ax2 = self.fig.add_subplot(212, sharex=self.ax1) 
        elif shy == True:
            self.ax2 = self.fig.add_subplot(212, sharey=self.ax1) 
        else:
            self.ax2 = self.fig.add_subplot(212)

        self.ax1.tick_params(axis='both', which='major', labelsize=8)
        self.ax2.tick_params(axis='both', which='major', labelsize=8)

        self.ax1.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
        self.ax2.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  

        self.ax1.yaxis.get_offset_text().set_size(8) 
        self.ax2.yaxis.get_offset_text().set_size(8) 

    def reAx3(self, shx=True, shy=True):

        try:
            self.fig.clear()
        except:
            pass

        try:
            self.ax1.clear() 
            self.delaxes(self.ax1) #.clear()
        except:
            pass
            
        try:
            self.ax2.clear() 
            self.delaxes(self.ax2) #.clear()
        except:
            pass
        
        try:    
            self.ax3.clear() 
            self.delaxes(self.ax3) #.clear()
        except:
            pass

        self.ax1 = self.fig.add_subplot(211)
        if shx and shy:
            self.ax2 = self.fig.add_subplot(212, sharex=self.ax1, sharey=self.ax1)
        elif shx:
            self.ax2 = self.fig.add_subplot(212, sharex=self.ax1) 
        elif shy:
            self.ax2 = self.fig.add_subplot(212, sharey=self.ax1) 
        else:
            self.ax2 = self.fig.add_subplot(212) 

        self.ax3 = self.ax1.twinx()

        self.ax1.tick_params(axis='both', which='major', labelsize=8)
        self.ax2.tick_params(axis='both', which='major', labelsize=8)
        self.ax3.tick_params(axis='both', which='major', labelsize=8)

        self.ax1.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
        self.ax2.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
        self.ax3.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  

        self.ax1.yaxis.get_offset_text().set_size(8) 
        self.ax2.yaxis.get_offset_text().set_size(8)    
        self.ax3.yaxis.get_offset_text().set_size(8)    
 
    def reAx4(self):

        try:
            self.fig.clear()
        except:
            pass

        # two main axes
        self.ax1 = self.fig.add_axes([0.15, 0.55,   0.625, 0.3672])
        self.ax2 = self.fig.add_axes([0.15, 0.135,  0.625, 0.3672])
        
        # for colourbars
        self.cax1 = self.fig.add_axes([0.8, 0.55,   0.025, 0.3672])
        self.cax2 = self.fig.add_axes([0.8, 0.135,  0.025, 0.3672])
        
        self.ax1.tick_params(axis='both', which='major', labelsize=8)
        self.ax2.tick_params(axis='both', which='major', labelsize=8)            if n == 2:
                self.ax3 = self.fig.add_subplot( 2, num, 3, sharex=self.ax1, sharey=self.ax1 )
                self.ax3.tick_params(axis='both', which='major', labelsize=8)
                self.ax3.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax3.yaxis.get_offset_text().set_size(8) 
                self.ax23 = self.fig.add_subplot( 2, num, num+3, sharex=self.ax21, sharey=self.ax21 )
                self.ax23.tick_params(axis='both', which='major', labelsize=8)
                self.ax23.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax23.yaxis.get_offset_text().set_size(8) 
            if n == 3:
                self.ax4 = self.fig.add_subplot( 2, num, 4, sharex=self.ax1, sharey=self.ax1 )
                self.ax4.tick_params(axis='both', which='major', labelsize=8)
                self.ax4.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax4.yaxis.get_offset_text().set_size(8) 
                self.ax24 = self.fig.add_subplot( 2, num, num+4, sharex=self.ax21, sharey=self.ax21 )
                self.ax24.tick_params(axis='both', which='major', labelsize=8)
                self.ax24.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax24.yaxis.get_offset_text().set_size(8) 
            if n == 4:
                self.ax5 = self.fig.add_subplot( 2, num, 5, sharex=self.ax1, sharey=self.ax1 )
                self.ax5.tick_params(axis='both', which='major', labelsize=8)
                self.ax5.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax5.yaxis.get_offset_text().set_size(8) 
                self.ax25 = self.fig.add_subplot( 2, num, num+5, sharex=self.ax21, sharey=self.ax21 )
                self.ax25.tick_params(axis='both', which='major', labelsize=8)
                self.ax25.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax25.yaxis.get_offset_text().set_size(8) 
            if n == 5:
                self.ax6 = self.fig.add_subplot( 2, num, 6, sharex=self.ax1, sharey=self.ax1 )
                self.ax6.tick_params(axis='both', which='major', labelsize=8)
                self.ax6.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax6.yaxis.get_offset_text().set_size(8) 
                self.ax26 = self.fig.add_subplot( 2, num, num+6, sharex=self.ax21, sharey=self.ax21 )
                self.ax26.tick_params(axis='both', which='major', labelsize=8)
                self.ax26.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
                self.ax26.yaxis.get_offset_text().set_size(8) 


        self.ax1.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
        self.ax2.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  

        self.ax1.yaxis.get_offset_text().set_size(8) 
        self.ax2.yaxis.get_offset_text().set_size(8) 

        self.cax1.tick_params(axis='both', which='major', labelsize=8)
        self.cax2.tick_params(axis='both', which='major', labelsize=8)

        self.cax1.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  
        self.cax2.ticklabel_format(style='sci', scilimits=(0,0), axis='y')  

        self.cax1.yaxis.get_offset_text().set_size(8) #.get_text()
        self.cax2.yaxis.get_offset_text().set_size(8) #.get_text()

        self.cax1.tick_params(labelsize=8) 
        self.cax2.tick_params(labelsize=8)


    def compute_initial_figure(self):
        
        t = np.arange(0,.3,1e-4)
        x = np.cos(t*2000.*np.pi*2)*np.exp(-t/.07)
        x2 = np.exp(-t/.07)
        dp = self.ax1.plot(t, x, 'r',label='test function')
        dp2 = self.ax2.plot(t, x2, 'r',label='test function2')
        self.ax1.set_xlabel("Time [s]", fontsize=8)
        self.ax1.set_ylabel("Signal [nV]", fontsize=8)

        self.ax1.tick_params(axis='both', which='major', labelsize=8)
        self.ax1.tick_params(axis='both', which='minor', labelsize=6)

        self.ax1.ticklabel_format(style='sci', scilimits=(0,0), axis='y') 
        self.ax1.legend(prop={'size':6})




import numpy as np
from numpy.linalg import lstsq
from numpy.linalg import norm
from numpy import fft 

import pylab

from scipy.signal import correlate

def autocorr(x):
    #result = np.correlate(x, x, mode='full')
    result = correlate(x, x, mode='full')
    return result[result.size/2:]

class AdaptiveFilter:

    def __init__(self, mu):
        self.mu = mu

    def adapt_filt_Ref(self, x, R, M, mu, PCA, lambda2=0.95, H0=0):
        """ Taken from .m file
        This function is written to allow the user to filter a input signal   
        with an adaptive filter that utilizes 2 reference signals instead of  
        the standard method which allows for only 1 reference signal.         
        Author: Rob Clemens              Date: 3/16/06                       
        Modified and ported to Python, now takes arbitray number of reference points  
        """
        #from akvo.tressel import pca 
        import akvo.tressel.pca as pca 

        if np.shape(x) != np.shape(R[0]): # or np.shape(x) != np.shape(rx1):
            print ("Error, non aligned")
            exit(1)
        
        if PCA == "Yes":
            print("Performing PCA calculation in noise cancellation")
            # PCA decomposition on ref channels so signals are less related
            R, K, means = pca.pca( R )
           
            # test for in loop reference  
            #print("Cull nearly zero terms?", np.shape(x), np.shape(R))     
            #R = R[0:3,:] 
            #R = R[2:4,:] 
            #print("    removed zero terms?", np.shape(x), np.shape(R))     
            #H0 = H0[0:3*np.shape(x)[0]]
            #H0 = H0[0:2*np.shape(x)[0]]

        if all(H0) == 0:
            H = np.zeros( (len(R)*M))
            #print ("resetting filter")
        else:
            H = H0

        Rn = np.ones(len(R)*M) / mu 
        
        r_ = np.zeros( (len(R), M) ) 
        e = np.zeros(len(x)) # error, desired output
        ilambda = lambda2**-1

        for z in range(0, len(x)):
            # Only look forwards, to avoid distorting the lates times 
            # (run backwards, if opposite and you don't care about distorting very late time.)
            for ir in range(len(R)):
                if z < M:
                    r_[ir,0:z] = R[ir][0:z]
                    r_[ir,z:M] = 0 
                else:
                    # TODO, use np.delete and np.append to speed this up
                    r_[ir,:] = R[ir][z-M:z]
            # reshape            
            r_n = np.reshape(r_, -1) #concatenate((r_v, r_h ))

            #K      = np.dot( np.diag(Rn,0), r_n) / (lambda2 + np.dot(r_n*Rn, r_n))  # Create/update K
            K      = (Rn* r_n) / (lambda2 + np.dot(r_n*Rn, r_n))  # Create/update K
            e[z]   = x[z] - np.dot(r_n.T, H)             # e is the filtered signal, input - r(n) * Filter Coefs
            H     += K*e[z];                             # Update Filter Coefficients
            Rn     = ilambda*Rn - ilambda*np.dot(np.dot(K, r_n.T), Rn)     # Update R(n)
        return e, H

    def transferFunctionFFT(self, D, R, reg=1e-2):
        from akvo.tressel import pca
        """
            Computes the transfer function (H) between a Data channel and 
            a number of Reference channels. The Matrices D and R are 
            expected to be in the frequency domain on input.
            | R1'R1 R1'R2 R1'R3|   |h1|   |R1'D|
            | R2'R1 R2'R2 R2'R3| * |h2| = |R2'D|
            | R3'R1 R3'R2 R3'R3|   |h3|   |R3'D|

            Returns the corrected array 
        """

        # PCA decomposition on ref channels so signals are less related
        #transMatrix, K, means = pca.pca( np.array([rx0, rx1]))   
        #RR = np.zeros(( np.shape(R[0])[0]*np.shape(R[0])[1], len(R)))
#         RR = np.zeros(( len(R), np.shape(R[0])[0]*np.shape(R[0])[1] ))
#         for ir in range(len(R)):
#             RR[ir,:] = np.reshape(R[ir], -1)
#         transMatrix, K, means = pca.pca(RR)    
#         #R rx0 = transMatrix[0,:]
#         # rx1 = transMatrix[1,:]
#         for ir in range(len(R)):
#             R[ir] = transMatrix[ir,0]

        import scipy.linalg 
        import akvo.tressel.pca as pca 
        # Compute as many transfer functions as len(R)
        # A*H = B
        nref = len(R)
        H = np.zeros( (np.shape(D)[1], len(R)), dtype=complex )
        for iw in range(np.shape(D)[1]):
            A = np.zeros( (nref, nref), dtype=complex )
            B = np.zeros( (nref) , dtype=complex)
            for ii in range(nref):
                for jj in range(nref):
                    # build A
                    A[ii,jj] = np.dot(R[ii][:,iw], R[jj][:,iw])                 

                # build B
                B[ii] = np.dot( R[ii][:,iw], D[:,iw] )

            # compute H(iw)
            #linalg.solve(a,b) if a is square
            #print "A", A
            #print "B", B
            # TODO, regularise this solve step? So as to not fit the spurious noise
            #print np.shape(B), np.shape(A) 
            #H[iw, :] = scipy.linalg.solve(A,B)
            H[iw, :] = scipy.linalg.lstsq(A,B,cond=reg)[0]
            #print "lstqt", np.shape(scipy.linalg.lstsq(A,B))
            #print "solve", scipy.linalg.solve(A,B)
            #H[iw,:]  = scipy.linalg.lstsq(A,B) # otherwise 
                #H = np.zeros( (np.shape(D)[1], )   )
        #print H #A, B
        Error = np.zeros(np.shape(D), dtype=complex)
        for ir in range(nref):
            for q in range( np.shape(D)[0] ):
                #print "dimcheck", np.shape(H[:,ir]), np.shape(R[ir][q,:] )
                Error[q,:] += H[:,ir]*R[ir][q,:]
        return D - Error

    def adapt_filt_tworefFreq(self, x, rx0, rx1, M, lambda2=0.95):
        """ Frequency domain version of above
        """
        from akvo.tressel import pca 

        pylab.figure()
        pylab.plot(rx0)
        pylab.plot(rx1)

        # PCA decomposition on ref channels so signals are less related
        transMatrix, K, means = pca.pca( np.array([rx0, rx1]))    
        rx0 = transMatrix[:,0]
        rx1 = transMatrix[:,1]
        
        pylab.plot(rx0)
        pylab.plot(rx1)
        pylab.show()
        exit()

        if np.shape(x) != np.shape(rx0) or np.shape(x) != np.shape(rx1):
            print ("Error, non aligned")
            exit(1)

        wx = fft.rfft(x)
        wr0 = fft.rfft(rx0)
        wr1 = fft.rfft(rx1)
 
        H = np.zeros( (2*M), dtype=complex ) 
        ident_mat = np.eye((2*M))
        Rn = ident_mat / 0.1
        r_v = np.zeros( (M), dtype=complex ) 
        r_h = np.zeros( (M), dtype=complex ) 
        e = np.zeros(len(x), dtype=complex )
        ilambda = lambda2**-1

        for z in range(0, len(wx)):
            # TODO Padd with Zeros or truncate if M >,< arrays 
            r_v = wr0[::-1][:M] 
            r_h = wr1[::-1][:M] 
            r_n = np.concatenate((r_v, r_h ))
            K      = np.dot(Rn, r_n) / (lambda2 + np.dot(np.dot(r_n.T, Rn), r_n))  # Create/update K
            e[z]   = wx[z] - np.dot(r_n,H)        # e is the filtered signal, input - r(n) * Filter Coefs
            H     += K * e[z];                    # Update Filter Coefficients
            Rn     = ilambda*Rn - ilambda*K*r_n.T*Rn  # Update R(n)
        
        return fft.irfft(e)

    def iter_filt_refFreq(self, x, rx0, Ahat=.05, Bhat=.5, k=0.05):

        X = np.fft.rfft(x)
        X0 = np.copy(X)
        RX0 = np.fft.rfft(rx0)

        # step 0
        Abs2HW = []
        alphai =  k * (np.abs(Ahat)**2 / np.abs(Bhat)**2) 
        betai  =  k * (1. / (np.abs(Bhat)**2) ) 
        Hw     =  ((1.+alphai) * np.abs(X)**2 ) / (np.abs(X)**2 + betai*(np.abs(RX0)**2))
        H      =  np.abs(Hw)**2
        pylab.ion()
        pylab.figure()
        for i in range(10):
            #print "alphai", alphai
            #print "betai", betai
            #print "Hw", Hw
            alphai = k * (np.abs(Ahat)**2 / np.abs(Bhat)**2) * np.product(H, axis=0)
            betai  = k * (1. / np.abs(Bhat)**2) * np.product(H, axis=0)
            # update signal
            Hw   =  ((1.+alphai) * np.abs(X)**2) / (np.abs(X)**2 + betai*np.abs(RX0)**2)
            Hw = np.nan_to_num(Hw)
            X *= Hw
            H = np.vstack( (H, np.abs(Hw)**2) )
            #print "Hw", Hw
            pylab.cla()
            pylab.plot(Hw)
            #pylab.plot(np.abs(X))
            #pylab.plot(np.abs(RX0))
            pylab.draw()
            raw_input("wait")

        pylab.cla()
        pylab.ioff()
        #return np.fft.irfft(X0-X)
        return np.fft.irfft(X)

    def iter_filt_refFreq(self, x, rx0, rx1, Ahat=.1, Bhat=1., k=0.001):

        X = np.fft.rfft(x)
        X0 = np.copy(X)
        RX0 = np.fft.rfft(rx0)
        RX1 = np.fft.rfft(rx1)

        # step 0
        alphai =  k * (np.abs(Ahat)**2 / np.abs(Bhat)**2) 
        betai  =  k * (1. / (np.abs(Bhat)**2) ) 
        #Hw     =  ((1.+alphai) * np.abs(X)**2 ) / (np.abs(X)**2 + betai*(np.abs(RX0)**2))
        H      =  np.ones(len(X)) # abs(Hw)**2
        #pylab.ion()
        #pylab.figure(334)
        for i in range(1000):
            #print "alphai", alphai
            #print "betai", betai
            #print "Hw", Hw
            alphai = k * (np.abs(Ahat)**2 / np.abs(Bhat)**2) * np.product(H, axis=0)
            betai  = k * (1. / np.abs(Bhat)**2) * np.product(H, axis=0)
            # update signal
            Hw   =  ((1.+alphai) * np.abs(X)**2) / (np.abs(X)**2 + betai*np.abs(RX0)**2)
            Hw = np.nan_to_num(Hw)
            X *= Hw #.conjugate
            #H = np.vstack((H, np.abs(Hw)**2) )
            H = np.vstack((H, np.abs(Hw)) )
            #print "Hw", Hw
            #pylab.cla()
            #pylab.plot(Hw)
            #pylab.plot(np.abs(X))
            #pylab.plot(np.abs(RX0))
            #pylab.draw()
            #raw_input("wait")

        #pylab.cla()
        #pylab.ioff()
        return np.fft.irfft(X0-X)
        #return np.fft.irfft(X)

    def Tdomain_DFT(self, desired, input, S):
        """ Lifted from Adaptive filtering toolbox. Modefied to accept more than one input 
            vector
        """
        
        # Initialisation Procedure
        nCoefficients =   S["filterOrderNo"]/2+1
        nIterations   =   len(desired)

        # Pre Allocations
        errorVector  = np.zeros(nIterations, dtype='complex')
        outputVector = np.zeros(nIterations, dtype='complex')
        
        # Initial State
        coefficientVectorDFT =   np.fft.rfft(S["initialCoefficients"])/np.sqrt(float(nCoefficients))
        desiredDFT           =   np.fft.rfft(desired)
        powerVector          =   S["initialPower"]*np.ones(nCoefficients)

        # Improve source code regularity, pad with zeros
        # TODO, confirm zeros(nCoeffics) not nCoeffics-1
        prefixedInput  =   np.concatenate([np.zeros(nCoefficients-1), np.array(input)])

        # Body
        pylab.ion()
        pylab.figure(11)
        for it in range(nIterations): # = 1:nIterations,
            
            regressorDFT = np.fft.rfft(prefixedInput[it:it+nCoefficients][::-1]) /\
                           np.sqrt(float(nCoefficients))

            # Summing two column vectors
            powerVector = S["alpha"] * (regressorDFT*np.conjugate(regressorDFT)) + \
                                  (1.-S["alpha"])*(powerVector)

            pylab.cla()
            #pylab.plot(prefixedInput[::-1], 'b')
            #pylab.plot(prefixedInput[it:it+nCoefficients][::-1], 'g', linewidth=3)
            #pylab.plot(regressorDFT.real)
            #pylab.plot(regressorDFT.imag)
            pylab.plot(powerVector.real)
            pylab.plot(powerVector.imag)
            #pylab.plot(outputVector)
            #pylab.plot(errorVector.real)
            #pylab.plot(errorVector.imag)
            pylab.draw()
            #raw_input("wait")

            outputVector[it] = np.dot(coefficientVectorDFT.T, regressorDFT)

            #errorVector[it] = desired[it] - outputVector[it]
            errorVector[it] = desiredDFT[it] - outputVector[it]

            #print errorVector[it], desired[it], outputVector[it]

            # Vectorized
            coefficientVectorDFT += (S["step"]*np.conjugate(errorVector[it])*regressorDFT) /\
                                    (S['gamma']+powerVector)

        return np.real(np.fft.irfft(errorVector))
        #coefficientVector = ifft(coefficientVectorDFT)*sqrt(nCoefficients);

    def Tdomain_DCT(self, desired, input, S):
        """ Lifted from Adaptive filtering toolbox. Modefied to accept more than one input 
            vector. Uses cosine transform
        """
        from scipy.fftpack import dct
 
        # Initialisation Procedure
        nCoefficients =   S["filterOrderNo"]+1
        nIterations   =   len(desired)

        # Pre Allocations
        errorVector  = np.zeros(nIterations)
        outputVector = np.zeros(nIterations)
        
        # Initial State
        coefficientVectorDCT =   dct(S["initialCoefficients"]) #/np.sqrt(float(nCoefficients))
        desiredDCT           =   dct(desired)
        powerVector          =   S["initialPower"]*np.ones(nCoefficients)

        # Improve source code regularity, pad with zeros
        prefixedInput  =   np.concatenate([np.zeros(nCoefficients-1), np.array(input)])
        
        # Body
        #pylab.figure(11)
        #pylab.ion()
        for it in range(0, nIterations): # = 1:nIterations,
            
            regressorDCT = dct(prefixedInput[it:it+nCoefficients][::-1], type=2) 
            #regressorDCT = dct(prefixedInput[it+nCoefficients:it+nCoefficients*2+1])#[::-1]) 

            # Summing two column vectors
            powerVector = S["alpha"]*(regressorDCT) + (1.-S["alpha"])*(powerVector)
            #pylab.cla()
            #pylab.plot(powerVector)
            #pylab.draw()

            outputVector[it] = np.dot(coefficientVectorDCT.T, regressorDCT)
            #errorVector[it] = desired[it] - outputVector[it]
            errorVector[it] = desiredDCT[it] - outputVector[it]

            # Vectorized
            coefficientVectorDCT += (S["step"]*errorVector[it]*regressorDCT) #/\
                                    #(S['gamma']+powerVector)

        #pylab.plot(errorVector)
        #pylab.show()
        return dct(errorVector, type=3)
        #coefficientVector = ifft(coefficientVectorDCT)*sqrt(nCoefficients);



    def Tdomain_CORR(self, desired, input, S):

        from scipy.linalg import toeplitz
        from scipy.signal import correlate

        # Autocorrelation
        ac = np.correlate(input, input, mode='full')
        ac = ac[ac.size/2:]
        R = toeplitz(ac)
        
        # cross correllation
        r = np.correlate(desired, input, mode='full')
        r = r[r.size/2:]
        
        #r = np.correlate(desired, input, mode='valid')
        print ("R", np.shape(R))
        print ("r", np.shape(r))
        print ("solving")
        #H = np.linalg.solve(R,r)
        H = np.linalg.lstsq(R,r,rcond=.01)[0]
        #return desired - np.dot(H,input)
        print ("done solving")
        pylab.figure()
        pylab.plot(H)
        pylab.title("H")
        #return desired - np.convolve(H, input, mode='valid')
        #return desired - np.convolve(H, input, mode='same')
        #return np.convolve(H, input, mode='same')
        return desired - np.dot(toeplitz(H), input)
        #return np.dot(R, H)

#         T = toeplitz(input)
#         print "shapes", np.shape(T), np.shape(desired)
#         h = np.linalg.lstsq(T, desired)[0]
#         print "shapes", np.shape(h), np.shape(input)
#         #return np.convolve(h, input, mode='same')
#         return desired - np.dot(T,h)
 
    def Fdomain_CORR(self, desired, input, dt, freq):
        
        from scipy.linalg import toeplitz
        
        # Fourier domain
        Input = np.fft.rfft(input)
        Desired = np.fft.rfft(desired)

        T = toeplitz(Input)
        #H = np.linalg.solve(T, Desired)
        H = np.linalg.lstsq(T, Desired)[0]
#         ac = np.correlate(Input, Input, mode='full')
#         ac = ac[ac.size/2:]
#         R = toeplitz(ac)
#         
#         r = np.correlate(Desired, Input, mode='full')
#         r = r[r.size/2:]
#         
#         #r = np.correlate(desired, input, mode='valid')
#         print "R", np.shape(R)
#         print "r", np.shape(r)
#         print "solving"
#         H = np.linalg.solve(R,r)
#         #H = np.linalg.lstsq(R,r)
#         #return desired - np.dot(H,input)
#         print "done solving"
        pylab.figure()
        pylab.plot(H.real)
        pylab.plot(H.imag)
        pylab.plot(Input.real)
        pylab.plot(Input.imag)
        pylab.plot(Desired.real)
        pylab.plot(Desired.imag)
        pylab.legend(["hr","hi","ir","ii","dr","di"])
        pylab.title("H")
        #return desired - np.fft.irfft(Input*H)
        return np.fft.irfft(H*Input)

    def Tdomain_RLS(self, desired, input, S):
        """
            A DFT is first performed on the data. Than a RLS algorithm is carried out 
            for noise cancellation. Related to the RLS_Alt Algoritm 5.3 in  Diniz book.
            The desired and input signals are assummed to be real time series data.
        """

        # Transform data into frequency domain
        Input = np.fft.rfft(input)
        Desired = np.fft.rfft(desired)

        # Initialisation Procedure
        nCoefficients = S["filterOrderNo"]+1
        nIterations   = len(Desired)

        # Pre Allocations
        errorVector  = np.zeros(nIterations, dtype="complex")
        outputVector = np.zeros(nIterations, dtype="complex")
        errorVectorPost  = np.zeros(nIterations, dtype="complex")
        outputVectorPost = np.zeros(nIterations, dtype="complex")
        coefficientVector = np.zeros( (nCoefficients, nIterations+1), dtype="complex" )        

        # Initial State
        coefficientVector[:,1] = S["initialCoefficients"]  
        S_d                    = S["delta"]*np.eye(nCoefficients)

        # Improve source code regularity, pad with zeros
        prefixedInput = np.concatenate([np.zeros(nCoefficients-1, dtype="complex"), 
                                np.array(Input)])
        invLambda = 1./S["lambda"]
        
        # Body
        pylab.ion()
        pylab.figure(11)

        for it in range(nIterations):
            
            regressor = prefixedInput[it:it+nCoefficients][::-1]

            # a priori estimated output
            outputVector[it] = np.dot(coefficientVector[:,it].T, regressor)
       
            # a priori error
            errorVector[it] = Desired[it] - outputVector[it]

            psi             = np.dot(S_d, regressor)
            if np.isnan(psi).any():
                print ("psi", psi)
                exit(1)
            
            pylab.cla()
            #pylab.plot(psi)
            pylab.plot(regressor.real)
            pylab.plot(regressor.imag)
            pylab.plot(coefficientVector[:,it].real)
            pylab.plot(coefficientVector[:,it].imag)
            pylab.legend(["rr","ri", "cr", "ci"])
            pylab.draw()
            raw_input("paws")

            S_d             = invLambda * (S_d - np.dot(psi, psi.T)  /\
                                S["lambda"] + np.dot(psi.T, regressor))

            coefficientVector[:,it+1] = coefficientVector[:,it] + \
                                        np.conjugate(errorVector[it])*np.dot(S_d, regressor)
            # A posteriori estimated output
            outputVectorPost[it]  =  np.dot(coefficientVector[:,it+1].T, regressor)

            # A posteriori error
            errorVectorPost[it] = Desired[it] - outputVectorPost[it]
 
        errorVectorPost = np.nan_to_num(errorVectorPost)

        pylab.figure(11)
        print (np.shape(errorVectorPost))
        pylab.plot(errorVectorPost.real)
        pylab.plot(errorVectorPost.imag)
        pylab.show()
        print(errorVectorPost)
        #return np.fft.irfft(Desired)
        return np.fft.irfft(errorVectorPost)

if __name__ == "__main__":

    def noise(nu, t, phi):
        return np.sin(nu*2.*np.pi*t + phi)

    import matplotlib.pyplot as plt
    print("Test driver for adaptive filtering")
    Filt = AdaptiveFilter(.1)
    t = np.arange(0, .5, 1e-4)
    omega = 2000 * 2.*np.pi
    T2 = .100
    n1 = noise(60, t, .2   )
    n2 = noise(61, t, .514 )
    x = np.sin(omega*t)* np.exp(-t/T2) + 2.3*noise(60, t, .34) + 1.783*noise(31, t, 2.1)
    e = Filt.adapt_filt_tworef(x, n1, n2, 200, .98)
    plt.plot(t,  x)
    plt.plot(t, n1)
    plt.plot(t, n2)
    plt.plot(t,  e)
    plt.show()



import numpy, array #,rpy2
from matplotlib import pyplot as plt
import numpy as np
from scipy.optimize import least_squares
from rpy2.robjects.packages import importr

import rpy2.robjects as robjects
import rpy2.robjects.numpy2ri

#import notch
from numpy.fft import fft, fftfreq

# We know/can calculate frequency peak, use this to guess where picks will be.
# maybe have a sliding window that reports peak values.
def peakPicker(data, omega, dt):

    # compute window based on omega and dt
    # make sure you are not aliased, grab every other peak
    window = (2*numpy.pi) / (omega*dt)

    data = numpy.array(data) 
    peaks = []
    troughs = []
    times = []
    times2 = []
    indices = []
    ws = 0
    we = window
    ii = 0
    for i in range((int)(len(data)/window)):
        
        # initially was just returning this I think avg is better
        #times.append( (ws + numpy.abs(data[ws:we]).argmax()) * dt )
    
        peaks.append(numpy.max(data[ws:we]))
        times.append( (ws + data[ws:we].argmax()) * dt )
        indices.append( ii + data[ws:we].argmax() )        

        troughs.append(numpy.min(data[ws:we]))
        times2.append( (ws + (data[ws:we]).argmin()) * dt )
        indices.append( ii + data[ws:we].argmin() )        

        ws += window
        we += window
        ii += (int)(we-ws)
    
    #return numpy.array(peaks), numpy.array(times)
    
    # Averaging peaks does a good job of removing bias in noise
    return (numpy.array(peaks)-numpy.array(troughs))/2., \
        (numpy.array(times)+numpy.array(times2))/2., \
        indices           


#################################################
# Regress for T2 using rpy2 interface
def regressCurve(peaks,times,sigma2=1,intercept=True):

    # TODO, if regression fails, it might be because there is no exponential
    # term, maybe do a second regression then on a linear model. 
    b1  = 0                  # Bias
    b2  = 0                  # Linear 
    rT2 = 0.3                # T2 regressed
    r   = robjects.r         

    # Variable shared between R and Python
    robjects.globalenv['b1'] = b1
    robjects.globalenv['b2'] = b2
    robjects.globalenv['rT2'] = rT2
    robjects.globalenv['sigma2'] = sigma2
    value = robjects.FloatVector(peaks)
    times = robjects.FloatVector(numpy.array(times))
    
#    my_weights = robjects.RVector(value/sigma2)
#    robjects.globalenv['my_weigts'] = my_weights

#    if sigma2 != 0:
#        print "weighting"
#        tw = numpy.array(peaks)/sigma2 
#        my_weights = robjects.RVector( tw/numpy.max(tw) )
#    else:
#        my_weights = robjects.RVector(numpy.ones(len(peaks))) 

#    robjects.globalenv['my_weights'] = my_weights
    
    if (intercept):
        my_list = robjects.r('list(b1=50, b2=1e2, rT2=0.03)')
        my_lower = robjects.r('list(b1=0, b2=0, rT2=.005)')
        my_upper = robjects.r('list(b1=20000, b2=2000, rT2=.700)')
    else:
        my_list = robjects.r('list(b2=1e2, rT2=0.3)')
        my_lower = robjects.r('list(b2=0, rT2=.005)')
        my_upper = robjects.r('list(b2=2000, rT2=.700)')

    my_cont = robjects.r('nls.control(maxiter=1000, warnOnly=TRUE, printEval=FALSE)')

    
    if (intercept):
        #fmla = robjects.RFormula('value ~ b1 + exp(-times/rT2)')
        fmla = robjects.Formula('value ~ b1 + b2*exp(-times/rT2)')
        #fmla = robjects.RFormula('value ~ b1 + b2*times + exp(-times/rT2)')
    else:
        fmla = robjects.Formula('value ~ b2*exp(-times/rT2)')

    env = fmla.getenvironment()
    env['value'] = value
    env['times'] = times
    
    # ugly, but I get errors with everything else I've tried
    my_weights = robjects.r('rep(1,length(value))')
    for ii in range(len(my_weights)):
        my_weights[ii] *= peaks[ii]/sigma2
    Error = False
    #fit = robjects.r.nls(fmla,start=my_list,control=my_cont,weights=my_weights)
    if (sigma2 != 1):
        print("SIGMA 2")
        #fit = robjects.r.tryCatch(robjects.r.suppressWarnings(robjects.r.nls(fmla,start=my_list,control=my_cont,algorithm="port", \
        #                     weights=my_weights)), 'silent=TRUE')
        fit = robjects.r.tryCatch(robjects.r.nls(fmla,start=my_list,control=my_cont))#, \
                            # weights=my_weights))
    else:
        try:
            fit = robjects.r.tryCatch(robjects.r.nls(fmla,start=my_list,control=my_cont,algorithm="port"))#,lower=my_lower,upper=my_upper))
        except:
            print("regression issue pass")
            Error = True
    # If failure fall back on zero regression values   
    if not Error:
        #Error = fit[3][0]
        report =  r.summary(fit)
    b1 = 0
    b2 = 0 
    rT2 = 1
    if (intercept):
        if not Error:
            b1  =  r['$'](report,'par')[0]
            b2  =  r['$'](report,'par')[1]
            rT2 =  r['$'](report,'par')[2]
            #print  report
            #print  r['$'](report,'convergence')
            #print  r['convergence'] #(report,'convergence')
            #print  r['$'](report,'par')[13]
            #print  r['$'](report,'par')[14]
        else:
            print("ERROR DETECTED, regressed values set to default")
            b1 = 1e1
            b2 = 1e-2
            rT2 = 1e-2
            #print r['$'](report,'par')[0]
            #print r['$'](report,'par')[1]
            #print r['$'](report,'par')[2]
        return [b1,b2,rT2] 
    else:
        if not Error:
            rT2 =  r['$'](report,'par')[1]
            b2  =  r['$'](report,'par')[0]
        else:
            print("ERROR DETECTED, regressed values set to default")
        return [b2, rT2] 

#################################################
# Regress for T2 using rpy2 interface
def regressCurve2(peaks,times,sigma2=[None],intercept=True):

    if sigma2[0] != None:
        my_weights = robjects.FloatVector( sigma2 )

    # TODO, if regression fails, it might be because there is no exponential
    # term, maybe do a second regression then on a linear model. 
    b1  = 0                  # Bias
    b2  = 0                  # Linear 
    bb2  = 0                 # Linear 
    rT2 = 0.3                # T2 regressed
    rrT2 = 1.3               # T2 regressed
    r   = robjects.r         

    # Variable shared between R and Python
    robjects.globalenv['b1'] = b1
    robjects.globalenv['b2'] = b2
    robjects.globalenv['rT2'] = rT2
    
    robjects.globalenv['bb2'] = b2
    robjects.globalenv['rrT2'] = rT2
    
    #robjects.globalenv['sigma2'] = sigma2
    value = robjects.FloatVector(peaks)
    times = robjects.FloatVector(numpy.array(times))
    
    
    if (intercept):
        my_list = robjects.r('list(b1=.50, b2=1e2, rT2=0.03, bb2=1e1, rrT2=1.3)')
        my_lower = robjects.r('list(b1=0, b2=0, rT2=.005, bb2=0, rrT2=.005 )')
        my_upper = robjects.r('list(b1=2000, b2=2000, rT2=.700, bb2=2000, rrT2=1.3 )')
    else:
        my_list  = robjects.r('list(b2=.5, rT2=0.3,  bb2=.5, rrT2=1.3)')
        my_lower = robjects.r('list(b2=0,  rT2=.005, bb2=0,  rrT2=.005)')
        my_upper = robjects.r('list(b2=1,  rT2=2.6,    bb2=1,  rrT2=2.6)')

    my_cont = robjects.r('nls.control(maxiter=1000, warnOnly=TRUE, printEval=FALSE)')

    
    if (intercept):
        #fmla = robjects.RFormula('value ~ b1 + exp(-times/rT2)')
        fmla = robjects.Formula('value ~ b1 + b2*exp(-times/rT2) + bb2*exp(-times/rrT2)')
        #fmla = robjects.RFormula('value ~ b1 + b2*times + exp(-times/rT2)')
    else:
        fmla = robjects.Formula('value ~ b2*exp(-times/rT2) + bb2*exp(-times/rrT2)')

    env = fmla.getenvironment()
    env['value'] = value
    env['times'] = times
    
    # ugly, but I get errors with everything else I've tried
    Error = False
    #fit = robjects.r.nls(fmla,start=my_list,control=my_cont,weights=my_weights)
    if (sigma2[0] != None):
        #print("SIGMA 2")
        #fit = robjects.r.tryCatch(robjects.r.suppressWarnings(robjects.r.nls(fmla,start=my_list,control=my_cont,algorithm="port", \
        #                     weights=my_weights)), 'silent=TRUE')
        fit = robjects.r.tryCatch(robjects.r.nls(fmla,start=my_list,control=my_cont,algorithm='port',weights=my_weights,lower=my_lower,upper=my_upper))#, \
                            # weights=my_weights))
    else:
        try:
            fit = robjects.r.tryCatch(robjects.r.nls(fmla,start=my_list,control=my_cont,algorithm="port"))#,lower=my_lower,upper=my_upper))
        except:
            print("regression issue pass")
            Error = True
    # If failure fall back on zero regression values   
    if not Error:
        #Error = fit[3][0]
        report =  r.summary(fit)
    b1 = 0
    b2 = 0 
    rT2 = 1
    if (intercept):
        if not Error:
            b1  =  r['$'](report,'par')[0]
            b2  =  r['$'](report,'par')[1]
            rT2 =  r['$'](report,'par')[2]
            #print  report
            #print  r['$'](report,'convergence')
            #print  r['convergence'] #(report,'convergence')
            #print  r['$'](report,'par')[13]
            #print  r['$'](report,'par')[14]
        else:
            print("ERROR DETECTED, regressed values set to default")
            b1 = 1e1
            b2 = 1e-2
            rT2 = 1e-2
            #print r['$'](report,'par')[0]
            #print r['$'](report,'par')[1]
            #print r['$'](report,'par')[2]
        return [b1,b2,rT2, bb2, rrT2] 
    else:
        if not Error:
            rT2 =  r['$'](report,'par')[1]
            b2  =  r['$'](report,'par')[0]
            rrT2 =  r['$'](report,'par')[3]
            bb2  =  r['$'](report,'par')[2]
        else:
            print("ERROR DETECTED, regressed values set to default")
        return [b2, rT2, bb2, rrT2] 

def fun(x, t, y):
    """ Cost function for regression, single exponential, no DC term 
        x[0] = A0
        x[1] = zeta 
        x[2] = df
        x[3] = T2
    """
    # concatenated real and imaginary parts  
    pre =  np.concatenate((-x[0]*np.sin(2.*np.pi*x[2]*t + x[1])*np.exp(-t/x[3]), \
                           +x[0]*np.cos(2.*np.pi*x[2]*t + x[1])*np.exp(-t/x[3])))  
    return y-pre

def fun2(x, t, y):
    """ Cost function for regression, single exponential, no DC term 
        x[0] = A0
        x[1] = zeta 
        x[2] = T2
    """
    # concatenated real and imaginary parts  
    pre =  np.concatenate((x[0]*np.cos(x[1])*np.exp(-t/x[2]), \
                       -1.*x[0]*np.sin(x[1])*np.exp(-t/x[2])))  
    return y-pre


def quadratureDetect2(X, Y, tt, x0="None"): 
    """ Pure python quadrature detection using Scipy.  
        X = real part of NMR signal 
        Y = imaginary component of NMR signal 
        tt = time 
    """
    print("Pure Python Quad Det", "TODO look at loss functions and method")
    # Loss functions, linear, soft_l1, huber, cauchy, arctan 
    # df
    loss = 'cauchy'  #  'soft_l1'
    method = 'trf'   # trf, dogbox, lm 
    if x0=="None":
        x0 = np.array( [1., 0., 0., .2] ) # A0, zeta, df, T2 
        res_lsq = least_squares(fun, x0, args=(tt, np.concatenate((X, Y))), loss=loss, f_scale=1.0,\
            bounds=( [1., -np.pi, -5, .005] , [1000., np.pi, 5, .800] ),
            method=method 
            )
        x = res_lsq.x 
        print ("df", x[0], x[1], x[2], x[3])
    else:
        res_lsq = least_squares(fun, x0, args=(tt, np.concatenate((X, Y))), loss=loss, f_scale=1.0,\
            bounds=( [1., -np.pi, -5, .005] , [1000., np.pi, 5, .800] ),
            method=method 
            )

        #bounds=( [0., 0, -20, .0] , [1., np.pi, 20, .6] ))

    x = res_lsq.x 
    return res_lsq.success, x[0], x[2], x[1], x[3]
    
    # no df
    #x = np.array( [1., 0., 0.2] )
    #res_lsq = least_squares(fun2, x, args=(tt, np.concatenate((X, Y))), loss='soft_l1', f_scale=0.1)
    #x = res_lsq.x 
    #return conv, E0,df,phi,T2
    #return res_lsq.success, x[0], 0, x[1], x[2]

def quadratureDetect(X, Y, tt, CorrectFreq=False, BiExp=False, CorrectDC=False):
 
    r   = robjects.r        

    if CorrectDC:
        robjects.r(''' 
             Xc1 <- function(E01, df, tt, phi, T2_1, DC) {
	                DC + E01*cos(2*pi*df*tt + phi) * exp(-tt/T2_1)
            }
    
            Yc1 <- function(E01, df, tt, phi, T2_1, DC) {
	                DC - E01*sin(2*pi*df*tt + phi) * exp(-tt/T2_1)
            } 
            ''')
    else:   
        robjects.r(''' 
             Xc1 <- function(E01, df, tt, phi, T2_1) {
	                E01*cos(2*pi*df*tt + phi) * exp(-tt/T2_1)
            }
    
            Yc1 <- function(E01, df, tt, phi, T2_1) {
	                -E01*sin(2*pi*df*tt + phi) * exp(-tt/T2_1)
            } 
            ''')

    # bi-exponential 
    if CorrectDC:
        robjects.r(''' 
             Xc2 <- function(E01, E02, df, tt, phi, T2_1, T2_2, DC) {
	               DC + E01*cos(2*pi*df*tt + phi) * exp(-tt/T2_1) + 
	                DC + E02*cos(2*pi*df*tt + phi) * exp(-tt/T2_2)
            }

            Yc2 <- function(E01, E02, df, tt, phi, T2_1, T2_2, DC) {
	                DC - E01*sin(2*pi*df*tt + phi) * exp(-tt/T2_1) + 
	                DC - E02*sin(2*pi*df*tt + phi) * exp(-tt/T2_2)
            } 
            ''')
    else:   
        robjects.r(''' 
             Xc2 <- function(E01, E02, df, tt, phi, T2_1, T2_2) {
	               E01*cos(2*pi*df*tt + phi) * exp(-tt/T2_1) + 
	               E02*cos(2*pi*df*tt + phi) * exp(-tt/T2_2)
            }

            Yc2 <- function(E01, E02, df, tt, phi, T2_1, T2_2) {
	                -E01*sin(2*pi*df*tt + phi) * exp(-tt/T2_1) + 
	                -E02*sin(2*pi*df*tt + phi) * exp(-tt/T2_2)
            } 
            ''')

    # Make 0 vector 
    Zero = robjects.FloatVector(numpy.zeros(len(X)))
    
    # Fitted Parameters
    E01 = 0.
    E02 = 0.
    df = 0.
    phi = 0.
    T2_1 = 0.
    T2_2 = 0.
    DC = 0.
    robjects.globalenv['DC'] = DC
    robjects.globalenv['E01'] = E01
    robjects.globalenv['E02'] = E02
    robjects.globalenv['df'] = df
    robjects.globalenv['phi'] = phi
    robjects.globalenv['T2_1'] = T2_1
    robjects.globalenv['T2_2'] = T2_2
    XY = robjects.FloatVector(numpy.concatenate((X,Y)))
    
    # Arrays
    tt = robjects.FloatVector(numpy.array(tt))
    X = robjects.FloatVector(numpy.array(X))
    Y = robjects.FloatVector(numpy.array(Y))
    Zero = robjects.FloatVector(numpy.array(Zero))

    

    if BiExp:
        if CorrectDC:
            fmla = robjects.Formula('XY ~ c(Xc2( E01, E02, df, tt, phi, T2_1, T2_2, DC ), Yc2( E01, E02, df, tt, phi, T2_1, T2_2, DC ))')
            if CorrectFreq:    
                start = robjects.r('list(E01=.100, E02=.01,   df=0,    phi=0.    ,  T2_1=.100, T2_2=.01, DC=0.0)')
                lower = robjects.r('list(E01=1e-6, E02=1e-6,  df=-50,  phi=-3.14 ,  T2_1=.001, T2_2=.001, DC=0.0)')
                upper = robjects.r('list(E01=1.00, E02=1.0,   df=50,   phi=3.14  ,  T2_1=.800, T2_2=.8, DC=0.5)')
            else:
                start = robjects.r('list(E01=.100, E02=.01,   phi=0.9   ,  T2_1=.100, T2_2=.01,  DC=0.0)')
                lower = robjects.r('list(E01=1e-6, E02=1e-6,  phi=-3.14 ,  T2_1=.001, T2_2=.001, DC=0.0)')
                upper = robjects.r('list(E01=1.00, E02=1.0,   phi=3.14  ,  T2_1=.800, T2_2=.8,   DC=0.5)')
        else:
            fmla = robjects.Formula('XY ~ c(Xc2( E01, E02, df, tt, phi, T2_1, T2_2 ), Yc2( E01, E02, df, tt, phi, T2_1, T2_2))')
            if CorrectFreq:    
                start = robjects.r('list(E01=.100, E02=.01,   df=0,    phi=0.    ,  T2_1=.100, T2_2=.01)')
                lower = robjects.r('list(E01=1e-6, E02=1e-6,  df=-50,  phi=-3.14 ,  T2_1=.001, T2_2=.001)')
                upper = robjects.r('list(E01=1.00, E02=1.0,   df=50,   phi=3.14  ,  T2_1=.800, T2_2=.8)')
            else:
                start = robjects.r('list(E01=.100, E02=.01,   phi=0.9   ,  T2_1=.100, T2_2=.01)')
                lower = robjects.r('list(E01=1e-6, E02=1e-6,  phi=-3.14 ,  T2_1=.001, T2_2=.001)')
                upper = robjects.r('list(E01=1.00, E02=1.0,   phi=3.14  ,  T2_1=.800, T2_2=.8)')
    else: 
        if CorrectDC:
            fmla = robjects.Formula('XY ~ c(Xc1( E01, df, tt, phi, T2_1, DC), Yc1( E01, df, tt, phi, T2_1,DC))')
            if CorrectFreq:    
                start = robjects.r('list(E01=.100, df=0   , phi=0.   , T2_1=.100, DC=0.0)')
                lower = robjects.r('list(E01=1e-6, df=-50., phi=-3.14, T2_1=.001, DC=0.0)')
                upper = robjects.r('list(E01=1.00, df=50. , phi=3.14 , T2_1=.800, DC=0.5)')
            else:
                start = robjects.r('list(E01=.100, phi= 0.  , T2_1=.100, DC=0.0)')
                lower = robjects.r('list(E01=1e-6, phi=-3.13, T2_1=.001, DC=0.0)')
                upper = robjects.r('list(E01=1.00, phi= 3.13, T2_1=.800, DC=0.5)')
        else:
            fmla = robjects.Formula('XY ~ c(Xc1( E01, df, tt, phi, T2_1), Yc1( E01, df, tt, phi, T2_1))')
            if CorrectFreq:    
                start = robjects.r('list(E01=.100, df=0     , phi=0.   ,  T2_1=.100)')
                lower = robjects.r('list(E01=1e-6, df=-50. , phi=-3.14 ,  T2_1=.001)')
                upper = robjects.r('list(E01=1.00, df=50.  , phi=3.14  ,  T2_1=.800)')
            else:
                start = robjects.r('list(E01=.100, phi= 0.  , T2_1=.100)')
                lower = robjects.r('list(E01=1e-6, phi=-3.13, T2_1=.001)')
                upper = robjects.r('list(E01=1.00, phi= 3.13, T2_1=.800)')

    env = fmla.getenvironment()
    env['Zero'] = Zero
    env['X'] = X
    env['Y'] = Y
    env['XY'] = XY 
    env['tt'] = tt

    cont = robjects.r('nls.control(maxiter=10000, warnOnly=TRUE, printEval=FALSE)')
    
    fit = robjects.r.tryCatch(robjects.r.nls(fmla, start=start, control=cont, lower=lower, upper=upper, algorithm='port')) #, \
    #fit = robjects.r.tryCatch(robjects.r.nls(fmla, start=start, control=cont)) #, \
    report =  r.summary(fit)

    conv = r['$'](fit,'convergence')[0]
    #if conv:
    #    print (report)
    #    print ("conv", conv)
    print ("Conv",  r['$'](fit,'convergence'))  # T2
    print (report)
    
    if BiExp:
        if CorrectFreq:    
            E0   =  r['$'](report,'par')[0]   # E01
            E0  +=  r['$'](report,'par')[1]   # E02
            df  =  r['$'](report,'par')[2]   # offset
            phi =  r['$'](report,'par')[3]   # phase 
            T2  =  r['$'](report,'par')[4]   # T2
        else:
            E0   =  r['$'](report,'par')[0]   # E01
            E0  +=  r['$'](report,'par')[1]   # E02
            phi =  r['$'](report,'par')[2]   # phase 
            T2  =  r['$'](report,'par')[3]   # T2
    else:
        if CorrectFreq:    
            E0   =  r['$'](report,'par')[0]   # E01
            df  =  r['$'](report,'par')[1]   # offset
            phi =  r['$'](report,'par')[2]   # phase 
            T2  =  r['$'](report,'par')[3]   # T2
        else:
            E0   =  r['$'](report,'par')[0]   # E01
            phi =  r['$'](report,'par')[1]   # phase 
            T2  =  r['$'](report,'par')[2]   # T2
    #phi = 0.907655876627
    #phi = 0
    #print ("df", df)# = 0
    return conv, E0,df,phi,T2
    

#################################################
# Regress for T2 using rpy2 interface
def regressSpec(w, wL, X): #,sigma2=1,intercept=True):

    # compute s
    s = -1j*w

    # TODO, if regression fails, it might be because there is no exponential
    # term, maybe do a second regression then on a linear model. 
    a   = 0                  # Linear 
    rT2 = 0.1                # T2 regressed
    r   = robjects.r         

    # Variable shared between R and Python
    robjects.globalenv['a'] = a
    robjects.globalenv['rT2'] = rT2
    robjects.globalenv['wL'] = wL
    robjects.globalenv['nb'] = 0

    s = robjects.ComplexVector(numpy.array(s))
    XX = robjects.ComplexVector(X)
    Xr = robjects.FloatVector(numpy.real(X))
    Xi = robjects.FloatVector(numpy.imag(X))
    Xa = robjects.FloatVector(numpy.abs(X))
    Xri = robjects.FloatVector(numpy.concatenate((Xr,Xi)))
    
    #my_lower = robjects.r('list(a=.001, rT2=.001, nb=.0001)')
    my_lower = robjects.r('list(a=.001, rT2=.001)')
    #my_upper = robjects.r('list(a=1.5, rT2=.300, nb =100.)')
    my_upper = robjects.r('list(a=1.5, rT2=.300)')
     
    #my_list = robjects.r('list(a=.2, rT2=0.03, nb=.1)')
    my_list = robjects.r('list(a=.2, rT2=0.03)')
    my_cont = robjects.r('nls.control(maxiter=5000, warnOnly=TRUE, printEval=FALSE)')
    
    #fmla = robjects.Formula('Xri ~ c(a*Re((wL) / (wL^2+(s+1/rT2)^2 )), a*Im((wL)/(wL^2 + (s+1/rT2)^2 )))') # envelope
    ##fmla = robjects.Formula('Xri ~ c(a*Re((wL) / (wL^2+(s+1/rT2)^2 )), a*Im((wL)/(wL^2 + (s+1/rT2)^2 )))') # envelope
    #fmla = robjects.Formula('XX ~ a*(wL) / (wL^2 + (s+1/rT2)^2 )') # complex
    #fmla = robjects.Formula('Xa ~ abs(a*(wL) / (wL^2 + (s+1/rT2)^2 )) + nb') # complex
    fmla = robjects.Formula('Xa ~ abs(a*(wL) / (wL^2 + (s+1/rT2)^2 ))') # complex
 
    env = fmla.getenvironment()
    env['s'] = s
    env['Xr'] = Xr
    env['Xa'] = Xa
    env['Xi'] = Xi
    env['Xri'] = Xri
    env['XX'] = XX
     
    #fit = robjects.r.tryCatch(robjects.r.nls(fmla,start=my_list, control=my_cont)) #, lower=my_lower, algorithm='port')) #, \
    fit = robjects.r.tryCatch(robjects.r.nls(fmla, start=my_list, control=my_cont, lower=my_lower, upper=my_upper, algorithm='port')) #, \
    report =  r.summary(fit)
    #print report 
    #print  r.warnings()
 
    a  =  r['$'](report,'par')[0]
    rT2 =  r['$'](report,'par')[1]
    nb =  r['$'](report,'par')[2]
    
    return a, rT2, nb

#################################################
# Regress for T2 using rpy2 interface
def regressSpecComplex(w, wL, X): #,sigma2=1,intercept=True):

    # compute s
    s = -1j*w

    # TODO, if regression fails, it might be because there is no exponential
    # term, maybe do a second regression then on a linear model. 
    a   = 1                  # Linear 
    rT2 = 0.1                # T2 regressed
    r   = robjects.r         
    phi2 = 0                 # phase
    wL2 = wL

    # Variable shared between R and Python
    robjects.globalenv['a'] = a
    robjects.globalenv['rT2'] = rT2
    robjects.globalenv['wL'] = wL
    robjects.globalenv['wL2'] = 0
    robjects.globalenv['nb'] = 0
    robjects.globalenv['phi2'] = phi2

    s = robjects.ComplexVector(numpy.array(s))
    XX = robjects.ComplexVector(X)
    Xr = robjects.FloatVector(numpy.real(X))
    Xi = robjects.FloatVector(numpy.imag(X))
    Xa = robjects.FloatVector(numpy.abs(X))
    Xri = robjects.FloatVector(numpy.concatenate((X.real,X.imag)))

    robjects.r(''' 
        source('kernel.r')
    ''')   
    #Kw = robjects.globalenv['Kwri']
     
    #print (numpy.shape(X))
    
    #my_lower = robjects.r('list(a=.001, rT2=.001, nb=.0001)')
    #my_lower = robjects.r('list(a=.001, rT2=.001)') # Working
    my_lower = robjects.r('list(a=.001, rT2=.001, phi2=-3.14, wL2=wL-5)')
    #my_upper = robjects.r('list(a=1.5, rT2=.300, nb =100.)')
    my_upper = robjects.r('list(a=3.5, rT2=.300, phi2=3.14, wL2=wL+5)')
     
    #my_list = robjects.r('list(a=.2, rT2=0.03, nb=.1)')
    my_list = robjects.r('list(a=.2, rT2=0.03, phi2=0, wL2=wL)')
    my_cont = robjects.r('nls.control(maxiter=5000, warnOnly=TRUE, printEval=FALSE)')
    
    #fmla = robjects.Formula('Xri ~ c(a*Re((wL) / (wL^2+(s+1/rT2)^2 )), a*Im((wL)/(wL^2 + (s+1/rT2)^2 )))') # envelope
    #fmla = robjects.Formula('Xi   ~   Im(a*(sin(phi2)*s + ((1/rT2)*sin(phi2)) + wL*cos(phi2)) / (wL^2+(s+1/rT2)^2 ))') # envelope
    #fmla = robjects.Formula('Xri ~ c(Re(a*(sin(phi2)*s + ((1/rT2)*sin(phi2)) + wL*cos(phi2)) / (wL^2+(s+1/rT2)^2 )), Im(a*(sin(phi2)*s + ((1/rT2)*sin(phi2)) + wL*cos(phi2)) / (wL^2+(s+1/rT2)^2 )))') # envelope
    
    #fmlar = robjects.Formula('Xr ~ (Kwr(a, phi2, s, rT2, wL)) ') # envelope
    #fmlai = robjects.Formula('Xi ~ (Kwi(a, phi2, s, rT2, wL)) ') # envelope
    fmla = robjects.Formula('Xri ~ c(Kwr(a, phi2, s, rT2, wL2), Kwi(a, phi2, s, rT2, wL2) ) ') # envelope
    #fmla = robjects.Formula('Xri ~ (Kwri(a, phi2, s, rT2, wL)) ') # envelope
    
    #fmla = robjects.Formula('Xa ~ (abs(a*(sin(phi2)*s + ((1/rT2)*sin(phi2)) + wL*cos(phi2)) / (wL^2+(s+1/rT2)^2 )))') # envelope
    #fmla = robjects.Formula('XX ~ a*(wL) / (wL^2 + (s+1/rT2)^2 )') # complex
    #fmla = robjects.Formula('Xa ~ abs(a*(wL) / (wL^2 + (s+1/rT2)^2 )) + nb') # complex
    
    #fmla = robjects.Formula('Xri ~ c(a*Re((wL) / (wL^2+(s+1/rT2)^2 )), a*Im((wL)/(wL^2 + (s+1/rT2)^2 )))') # envelope
    
    #        self.Gw[iw, iT2] = ((np.sin(phi2) *  (alpha + 1j*self.w[iw]) + self.wL*np.cos(phi2)) / \
    #                               (self.wL**2 + (alpha+1.j*self.w[iw])**2 ))
    #        self.Gw[iw, iT2] = ds * self.sc*((np.sin(phi2)*( alpha + 1j*self.w[iw]) + self.wL*np.cos(phi2)) / \
    #                               (self.wL**2 + (alpha+1.j*self.w[iw])**2 ))
    
    # Works Amplitude Only!
    #fmla = robjects.Formula('Xa ~ abs(a*(wL) / (wL^2 + (s+1/rT2)^2 ))') # complex
 
    env = fmla.getenvironment()
    env['s'] = s
    env['Xr'] = Xr
    env['Xa'] = Xa
    env['Xi'] = Xi
    env['Xri'] = Xri
    env['XX'] = XX
     
    fit = robjects.r.tryCatch(robjects.r.nls(fmla,start=my_list, control=my_cont)) #, lower=my_lower, algorithm='port')) #, \
    #fitr = robjects.r.tryCatch(robjects.r.nls(fmlar, start=my_list, control=my_cont, lower=my_lower, upper=my_upper, algorithm='port')) #, \
    
    #env = fmlai.getenvironment()
    #fiti = robjects.r.tryCatch(robjects.r.nls(fmlai, start=my_list, control=my_cont, lower=my_lower, upper=my_upper, algorithm='port')) #, \
    
    #reportr =  r.summary(fitr)
    #reporti =  r.summary(fiti)
    report =  r.summary(fit)
    #print( report )
    #exit()
    #print( reportr )
    #print( reporti  )
    #exit()
    #print  r.warnings()
 
    #a   =  (r['$'](reportr,'par')[0] + r['$'](reporti,'par')[0]) / 2.
    #rT2 =  (r['$'](reportr,'par')[1] + r['$'](reporti,'par')[1]) / 2.
    #nb  =  (r['$'](reportr,'par')[2] + r['$'](reporti,'par')[2]) / 2.
    a   =  r['$'](report,'par')[0] 
    rT2 =  r['$'](report,'par')[1] 
    nb  =  r['$'](report,'par')[2] #phi2 

    print ("Python wL2", r['$'](report,'par')[3] )   
    print ("Python zeta", r['$'](report,'par')[2] )   
 
    return a, rT2, nb



###################################################################
###################################################################
###################################################################
if __name__ == "__main__":

    dt    = .0001
    T2    = .1
    omega = 2000.*2*numpy.pi
    phi   = .0
    T     = 8.*T2
    
    t = numpy.arange(0, T, dt)

    # Synthetic data, simple single decaying sinusoid 
    # with a single decay parameter and gaussian noise added 
    data = numpy.exp(-t/T2) * numpy.sin(omega * t + phi) + numpy.random.normal(0,.05,len(t)) \
                         + numpy.random.randint(-1,2,len(t))*numpy.random.exponential(.2,len(t)) 
    cdata = numpy.exp(-t/T2) * numpy.sin(omega * t + phi) #+ numpy.random.normal(0,.25,len(t))
    #data = numpy.random.normal(0,.25,len(t))

    sigma2 = numpy.std(data[::-len(data)/4])
    #sigma2 = numpy.var(data[::-len(data)/4])
    print("sigma2", sigma2)    
    
    [peaks,times,indices] = peakPicker(data, omega, dt)
    
    [b1,b2,rT2] = regressCurve(peaks,times)
    print("rT2 nonweighted", rT2)
    
    [b1,b2,rT2] = regressCurve(peaks,times,sigma2)
    print("rT2 weighted", rT2)

    envelope   =  numpy.exp(-t/T2)
    renvelope  =  numpy.exp(-t/rT2)

    #outf = file('regress.txt','w')
    #for i in range(len(times)):
    #    outf.write(str(times[i]) + "   " +  str(peaks[i]) + "\n")  
    #outf.close()

    plt.plot(t,data, 'b')
    plt.plot(t,cdata, 'g', linewidth=1)
    plt.plot(t,envelope, color='violet', linewidth=4)
    plt.plot(t,renvelope, 'r', linewidth=4)
    plt.plot(times, numpy.array(peaks), 'bo', markersize=8, alpha=.25)
    plt.legend(['noisy data','clean data','real envelope','regressed env','picks'])
    plt.savefig("regression.pdf")


    # FFT check
    fourier = fft(data)
    plt.figure()
    freq = fftfreq(len(data), d=dt)
    plt.plot(freq, (fourier.real))
    
    plt.show()

    # TODO do a bunch in batch mode to see if T2 estimate is better with or without 
    # weighting and which model is best.

    # TODO try with real data

    # TODO test filters (median, FFT, notch)

    # It looks like weighting is good for relatively low sigma, but for noisy data
    # it hurts us. Check



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main.ui'
#
# Created: Fri Jan 11 23:28:48 2013
#      by: PyQt4 UI code generator 4.9.6
#
# WARNING! All changes made in this file will be lost!




##########################################################
#
#
#
#
#
#from scipy import mean
import numpy

def pca(A, remove=[]):
    ''' The input matrix A should be a 2D numpy array in column-major 
        order. Each column is a dataset and PCA will be applied across
        columns
    '''
    #nrow = len(A[:,0])
    #ncol = len(A[0,:])
    nrow,ncol = numpy.shape(A)

    # Cast into matrix type for easier math
    A = numpy.matrix(A)

    # Allocate Covariance Matrix
    covMatrix = numpy.matrix(numpy.zeros((nrow, nrow)))
    
    # Compute Means of each row. Each column must be normalised
    meanArray = []
    for i in range(nrow):
        meanArray.append(numpy.mean(A[i].tolist()[0]) ) 
        A[i] -= meanArray[i]
    meanArray = numpy.array(meanArray) 
    
    # Generate Covariance Matrix
    covMatrix = numpy.cov(A)

    # Compute Eigen Values, Eigen Vectors     
    eigs = numpy.linalg.eig(covMatrix)
    K = eigs[1].T
    
    #print K
    #print "Eigen Values"
    #print eigs[0]
    
    # Zero requested components
    for i in remove:
        K[i] = numpy.zeros(len(K))        

    # Make Transform Matrices
    transMatrix = K*A
    
    # Return Necssary Stuff
    return numpy.array(transMatrix), K, meanArray
    #return K, A, meanArray 

#def invpca(K, A, means):
def invpca(transMatrix, K, means):
    '''Converts a PCA rotated dataset back to normal. Input parameters
    are the components to discard in re-creation.
    '''
    K = numpy.matrix(K)
    # Transform and Untransform the data
    #transMatrix = K*A
    untransMatrix = K.T*transMatrix
    
    # Correct for normalisation
    for i in range(len(transMatrix)):
        untransMatrix[i] += means[i]

    return untransMatrix    



import numpy, array #,rpy2
from matplotlib import pyplot as plt
import numpy as np
from scipy.optimize import least_squares

#from rpy2.robjects.packages import importr
#import rpy2.robjects as robjects
#import rpy2.robjects.numpy2ri

#import notch
from numpy.fft import fft, fftfreq

# We know/can calculate frequency peak, use this to guess where picks will be.
# maybe have a sliding window that reports peak values.
def peakPicker(data, omega, dt):

    # compute window based on omega and dt
    # make sure you are not aliased, grab every other peak
    window = (2*numpy.pi) / (omega*dt)

    data = numpy.array(data) 
    peaks = []
    troughs = []
    times = []
    times2 = []
    indices = []
    ws = 0
    we = window
    ii = 0
    for i in range((int)(len(data)/window)):
        
        # initially was just returning this I think avg is better
        #times.append( (ws + numpy.abs(data[ws:we]).argmax()) * dt )
    
        peaks.append(numpy.max(data[ws:we]))
        times.append( (ws + data[ws:we].argmax()) * dt )
        indices.append( ii + data[ws:we].argmax() )        

        troughs.append(numpy.min(data[ws:we]))
        times2.append( (ws + (data[ws:we]).argmin()) * dt )
        indices.append( ii + data[ws:we].argmin() )        

        ws += window
        we += window
        ii += (int)(we-ws)
    
    #return numpy.array(peaks), numpy.array(times)
    
    # Averaging peaks does a good job of removing bias in noise
    return (numpy.array(peaks)-numpy.array(troughs))/2., \
        (numpy.array(times)+numpy.array(times2))/2., \
        indices           


def fun(x, t, y):
    """ Cost function for regression, single exponential, no DC term 
        x[0] = A0
        x[1] = zeta 
        x[2] = df
        x[3] = T2
    """
    # concatenated real and imaginary parts  
    pre =  np.concatenate((-x[0]*np.sin(2.*np.pi*x[2]*t + x[1])*np.exp(-t/x[3]), \
                           +x[0]*np.cos(2.*np.pi*x[2]*t + x[1])*np.exp(-t/x[3])))  
    return y-pre

def fun2(x, t, y):
    """ Cost function for regression, single exponential, no DC term 
        x[0] = A0
        x[1] = zeta 
        x[2] = T2
    """
    # concatenated real and imaginary parts  
    pre =  np.concatenate((x[0]*np.cos(x[1])*np.exp(-t/x[2]), \
                       -1.*x[0]*np.sin(x[1])*np.exp(-t/x[2])))  
    return y-pre


def quadratureDetect2(X, Y, tt, x0="None"): 
    """ Pure python quadrature detection using Scipy.  
        X = real part of NMR signal 
        Y = imaginary component of NMR signal 
        tt = time 
    """
    print("Pure Python Quad Det", "TODO look at loss functions and method")
    # Loss functions, linear, soft_l1, huber, cauchy, arctan 
    # df
    loss = 'cauchy'  #  'soft_l1'
    method = 'trf'   # trf, dogbox, lm 
    if x0=="None":
        x0 = np.array( [1., 0., 0., .2] ) # A0, zeta, df, T2 
        res_lsq = least_squares(fun, x0, args=(tt, np.concatenate((X, Y))), loss=loss, f_scale=1.0,\
            bounds=( [1., -np.pi, -5, .005] , [1000., np.pi, 5, .800] ),
            method=method 
            )
        x = res_lsq.x 
        print ("df", x[0], x[1], x[2], x[3])
    else:
        res_lsq = least_squares(fun, x0, args=(tt, np.concatenate((X, Y))), loss=loss, f_scale=1.0,\
            bounds=( [1., -np.pi, -5, .005] , [1000., np.pi, 5, .800] ),
            method=method 
            )

        #bounds=( [0., 0, -20, .0] , [1., np.pi, 20, .6] ))

    x = res_lsq.x 
    return res_lsq.success, x[0], x[2], x[1], x[3]
    
    # no df
    #x = np.array( [1., 0., 0.2] )
    #res_lsq = least_squares(fun2, x, args=(tt, np.concatenate((X, Y))), loss='soft_l1', f_scale=0.1)
    #x = res_lsq.x 
    #return conv, E0,df,phi,T2
    #return res_lsq.success, x[0], 0, x[1], x[2]



###################################################################
###################################################################
###################################################################
if __name__ == "__main__":

    dt    = .0001
    T2    = .1
    omega = 2000.*2*numpy.pi
    phi   = .0
    T     = 8.*T2
    
    t = numpy.arange(0, T, dt)

    # Synthetic data, simple single decaying sinusoid 
    # with a single decay parameter and gaussian noise added 
    data = numpy.exp(-t/T2) * numpy.sin(omega * t + phi) + numpy.random.normal(0,.05,len(t)) \
                         + numpy.random.randint(-1,2,len(t))*numpy.random.exponential(.2,len(t)) 
    cdata = numpy.exp(-t/T2) * numpy.sin(omega * t + phi) #+ numpy.random.normal(0,.25,len(t))
    #data = numpy.random.normal(0,.25,len(t))

    sigma2 = numpy.std(data[::-len(data)/4])
    #sigma2 = numpy.var(data[::-len(data)/4])
    print("sigma2", sigma2)    
    
    [peaks,times,indices] = peakPicker(data, omega, dt)
    
    [b1,b2,rT2] = regressCurve(peaks,times)
    print("rT2 nonweighted", rT2)
    
    [b1,b2,rT2] = regressCurve(peaks,times,sigma2)
    print("rT2 weighted", rT2)

    envelope   =  numpy.exp(-t/T2)
    renvelope  =  numpy.exp(-t/rT2)

    #outf = file('regress.txt','w')
    #for i in range(len(times)):
    #    outf.write(str(times[i]) + "   " +  str(peaks[i]) + "\n")  
    #outf.close()

    plt.plot(t,data, 'b')
    plt.plot(t,cdata, 'g', linewidth=1)
    plt.plot(t,envelope, color='violet', linewidth=4)
    plt.plot(t,renvelope, 'r', linewidth=4)
    plt.plot(times, numpy.array(peaks), 'bo', markersize=8, alpha=.25)
    plt.legend(['noisy data','clean data','real envelope','regressed env','picks'])
    plt.savefig("regression.pdf")


    # FFT check
    fourier = fft(data)
    plt.figure()
    freq = fftfreq(len(data), d=dt)
    plt.plot(freq, (fourier.real))
    
    plt.show()

    # TODO do a bunch in batch mode to see if T2 estimate is better with or without 
    # weighting and which model is best.

    # TODO try with real data

    # TODO test filters (median, FFT, notch)

    # It looks like weighting is good for relatively low sigma, but for noisy data
    # it hurts us. Check
